MODULE wv_saturation
 IMPLICIT NONE
    INTEGER, PARAMETER :: r8 = SELECTED_REAL_KIND(15) ! Kind for 64-bits Real Numbers
    REAL(KIND=r8),PARAMETER :: SHR_CONST_MWWV   = 18.016_r8       ! molecular weight water vapor
    REAL(KIND=r8),PARAMETER :: SHR_CONST_MWDAIR = 28.966_r8       ! molecular weight dry air ~ kg/kmole
    REAL(KIND=r8),PARAMETER :: epsilo = shr_const_mwwv/shr_const_mwdair ! ratio of h2o to dry air molecular weights 
    REAL(KIND=r8),PARAMETER :: SHR_CONST_LATVAP = 2.501e6_r8      ! latent heat of evaporation ~ J/kg
    REAL(KIND=r8),PARAMETER :: SHR_CONST_TKFRZ  = 273.16_r8       ! freezing T of fresh water ~ K (intentionally made == to TKTRIP)
    REAL(KIND=r8),PARAMETER :: SHR_CONST_AVOGAD = 6.02214e26_r8   ! Avogadro's number ~ molecules/kmole
    REAL(KIND=r8),PARAMETER :: SHR_CONST_BOLTZ  = 1.38065e-23_r8  ! Boltzmann's constant ~ J/K/molecule
    REAL(KIND=r8),PARAMETER :: SHR_CONST_LATICE = 3.337e5_r8      ! latent heat of fusion ~ J/kg
    REAL(KIND=r8), PARAMETER :: latice = shr_const_latice ! Latent heat of fusion

    REAL(KIND=r8),PARAMETER :: trice  =  20.00_r8         ! Trans range from es over h2o to es over ice
    REAL(KIND=r8),PARAMETER :: ttrice=trice
    REAL(KIND=r8),PARAMETER :: SHR_CONST_CPDAIR = 1.00464e3_r8    ! specific heat of dry air ~ J/kg/K
    REAL(KIND=r8),PARAMETER :: cpair = shr_const_cpdair  ! specific heat of dry air (J/K/kg)
    REAL(KIND=r8),PARAMETER :: cp    =cpair
    REAL(KIND=r8),PARAMETER :: latvap = shr_const_latvap ! Latent heat of vaporization
    REAL(KIND=r8),PARAMETER :: SHR_CONST_RGAS   = SHR_CONST_AVOGAD*SHR_CONST_BOLTZ ! Universal gas constant ~ J/K/kmole
    REAL(KIND=r8),PARAMETER :: SHR_CONST_RWV    = SHR_CONST_RGAS/SHR_CONST_MWWV    ! Water vapor gas constant ~ J/K/kg
    INTEGER, PARAMETER :: plenest = 250! length of saturation vapor pressure table
    REAL(KIND=r8), PUBLIC, PARAMETER :: tmelt = shr_const_tkfrz   ! Freezing point of water

    REAL(KIND=r8)           :: estbl(plenest)      ! table values of saturation vapor pressure
    REAL(KIND=r8),PARAMETER :: tmn  = 173.16_r8          ! Minimum temperature entry in table
    REAL(KIND=r8),PARAMETER :: tmx  = 375.16_r8          ! Maximum temperature entry in table
    REAL(KIND=r8),PARAMETER :: tmin=tmn       ! min temperature (K) for table
    REAL(KIND=r8),PARAMETER :: tmax= tmx      ! max temperature (K) for table
    LOGICAL ,PARAMETER :: icephs=.TRUE.  ! false => saturation vapor press over water only
    INTEGER ,PARAMETER ::  iterp =2             ! #iterations for precipitation calculation
    INTEGER  ::  k1mb  =1  ! index of the eta level near 1 mb
    REAL(KIND=r8)           :: pcf(6)     ! polynomial coeffs -> es transition water to ice
    REAL(KIND=r8), PRIVATE :: hlatf  = latice
    REAL(KIND=r8), PRIVATE :: hlatv  = latvap
    REAL(KIND=r8), PRIVATE :: rgasv  = SHR_CONST_RWV    ! Gas constant for water vapor
    REAL(KIND=r8), PRIVATE :: t0 = tmelt                ! approximate freezing temp

CONTAINS

  SUBROUTINE findsp (nCols,pver, q, t, p, tsp, qsp)
    IMPLICIT NONE
    !----------------------------------------------------------------------- 
    ! 
    ! Purpose: 
    !     find the wet bulb temperature for a given t and q
    !     in a longitude height section
    !     wet bulb temp is the temperature and spec humidity that is 
    !     just saturated and has the same enthalpy
    !     if q > qs(t) then tsp > t and qsp = qs(tsp) < q
    !     if q < qs(t) then tsp < t and qsp = qs(tsp) > q
    !
    ! Method: 
    ! a Newton method is used
    ! first guess uses an algorithm provided by John Petch from the UKMO
    ! we exclude points where the physical situation is unrealistic
    ! e.g. where the temperature is outside the range of validity for the
    !      saturation vapor pressure, or where the water vapor pressure
    !      exceeds the ambient pressure, or the saturation specific humidity is 
    !      unrealistic
    ! 
    ! Author: P. Rasch
    ! 
    !-----------------------------------------------------------------------
    !
    !     input arguments
    !
    INTEGER, INTENT(in) :: nCols                 ! number of columns (max)
    INTEGER, INTENT(in) :: pver                  ! number of vertical levels

    REAL(KIND=r8), INTENT(in) :: q(nCols,pver)        ! water vapor (kg/kg)
    REAL(KIND=r8), INTENT(in) :: t(nCols,pver)        ! temperature (K)
    REAL(KIND=r8), INTENT(in) :: p(nCols,pver)        ! pressure    (Pa)
    !
    ! output arguments
    !
    REAL(KIND=r8), INTENT(out) :: tsp(nCols,pver)      ! saturation temp (K)
    REAL(KIND=r8), INTENT(out) :: qsp(nCols,pver)      ! saturation mixing ratio (kg/kg)
    !
    ! local variables
    !
    INTEGER i                 ! work variable
    INTEGER k                 ! work variable
    LOGICAL lflg              ! work variable
    INTEGER iter              ! work variable
    INTEGER l                 ! work variable
    LOGICAL :: error_found

    REAL(KIND=r8) omeps                ! 1 minus epsilon
    REAL(KIND=r8) trinv                ! work variable
    REAL(KIND=r8) es                   ! sat. vapor pressure
    REAL(KIND=r8) desdt                ! change in sat vap pressure wrt temperature
    !     real(KIND=r8) desdp                ! change in sat vap pressure wrt pressure
    REAL(KIND=r8) dqsdt                ! change in sat spec. hum. wrt temperature
    REAL(KIND=r8) dgdt                 ! work variable
    REAL(KIND=r8) g                    ! work variable
    REAL(KIND=r8) weight(nCols)        ! work variable
    REAL(KIND=r8) hlatsb               ! (sublimation)
    REAL(KIND=r8) hlatvp               ! (vaporization)
    REAL(KIND=r8) hltalt(nCols,pver)   ! lat. heat. of vap.
    REAL(KIND=r8) tterm                ! work var.
    REAL(KIND=r8) qs                   ! spec. hum. of water vapor
    REAL(KIND=r8) tc                   ! crit temp of transition to ice

    ! work variables
    REAL(KIND=r8) t1, q1, dt, dq
    REAL(KIND=r8) dtm, dqm
    REAL(KIND=r8) qvd, a1, tmp
    REAL(KIND=r8) rair
    REAL(KIND=r8) r1b, c1, c2, c3
    REAL(KIND=r8) denom
    REAL(KIND=r8) dttol
    REAL(KIND=r8) dqtol
    INTEGER doit(nCols) 
    REAL(KIND=r8) enin(nCols), enout(nCols)
    REAL(KIND=r8) tlim(nCols)
    REAL(KIND=r8) epsqs
    epsqs = epsilo
    k1mb=1
    omeps = 1.0_r8 - epsqs
    trinv = 1.0_r8/ttrice
    a1 = 7.5_r8*LOG(10.0_r8)
    rair =  287.04_r8
    c3 = rair*a1/cp
    dtm = 0.0_r8    ! needed for iter=0 blowup with f90 -ei
    dqm = 0.0_r8    ! needed for iter=0 blowup with f90 -ei
    dttol = 1.e-4_r8 ! the relative temp error tolerance required to quit the iteration
    dqtol = 1.e-4_r8 ! the relative moisture error tolerance required to quit the iteration
    !  tmin = 173.16_r8 ! the coldest temperature we can deal with
    !
    ! max number of times to iterate the calculation
    iter = 50
    !
    DO k = k1mb,pver

       !
       ! first guess on the wet bulb temperature
       !
       DO i = 1,ncols

          ! limit the temperature range to that relevant to the sat vap pres tables

          tlim(i) = MIN(MAX(t(i,k),173.0_r8),373.0_r8)

          es = estblf(tlim(i))
          denom = p(i,k) - omeps*es
          qs = epsqs*es/denom
          doit(i) = 0
          enout(i) = 1.0_r8
          ! make sure a meaningful calculation is possible
          IF (p(i,k) > 5.0_r8*es .AND. qs > 0.0_r8 .AND. qs < 0.5_r8) THEN
             !
             ! Saturation specific humidity
             !
             qs = MIN(epsqs*es/denom,1.0_r8)
             !
             ! "generalized" analytic expression for t derivative of es
             !  accurate to within 1 percent for 173.16 < t < 373.16
             !
             ! Weighting of hlat accounts for transition from water to ice
             ! polynomial expression approximates difference between es over
             ! water and es over ice from 0 to -ttrice (C) (min of ttrice is
             ! -40): required for accurate estimate of es derivative in transition
             ! range from ice to water also accounting for change of hlatv with t
             ! above freezing where const slope is given by -2369 j/(kg c) = cpv - cw
             !
             tc     = tlim(i) - t0
             lflg   = (tc >= -ttrice .AND. tc < 0.0_r8)
             weight(i) = MIN(-tc*trinv,1.0_r8)
             hlatsb = hlatv + weight(i)*hlatf
             hlatvp = hlatv - 2369.0_r8*tc
             IF (tlim(i) < t0) THEN
                hltalt(i,k) = hlatsb
             ELSE
                hltalt(i,k) = hlatvp
             END IF
             enin(i) = cp*tlim(i) + hltalt(i,k)*q(i,k)

             ! make a guess at the wet bulb temp using a UKMO algorithm (from J. Petch)
             tmp =  q(i,k) - qs
             c1 = hltalt(i,k)*c3
             c2 = (tlim(i) + 36.0_r8)**2
             r1b    = c2/(c2 + c1*qs)
             qvd   = r1b*tmp
             tsp(i,k) = tlim(i) + ((hltalt(i,k)/cp)*qvd)
             !#ifdef DEBUG
             !             if ( (lchnk == lchnklook(nlook) ) .and. (i == icollook(nlook) ) ) then
             !                write (6,*) ' relative humidity ', q(i,k)/qs
             !                write (6,*) ' first guess ', tsp(i,k)
             !             endif
             !#endif
             es = estblf(tsp(i,k))
             qsp(i,k) = MIN(epsqs*es/(p(i,k) - omeps*es),1.0_r8)
          ELSE
             doit(i) = 1
             tsp(i,k) = tlim(i)
             qsp(i,k) = q(i,k)
             enin(i) = 1.0_r8
          ENDIF
       END DO   ! end do i
       !
       ! now iterate on first guess
       !
       DO l = 1, iter
          dtm = 0.0_r8
          dqm = 0.0_r8
          DO i = 1,ncols
             IF (doit(i) == 0) THEN
                es = estblf(tsp(i,k))
                !
                ! Saturation specific humidity
                !
                qs = MIN(epsqs*es/(p(i,k) - omeps*es),1.0_r8)
                !
                ! "generalized" analytic expression for t derivative of es
                ! accurate to within 1 percent for 173.16 < t < 373.16
                !
                ! Weighting of hlat accounts for transition from water to ice
                ! polynomial expression approximates difference between es over
                ! water and es over ice from 0 to -ttrice (C) (min of ttrice is
                ! -40): required for accurate estimate of es derivative in transition
                ! range from ice to water also accounting for change of hlatv with t
                ! above freezing where const slope is given by -2369 j/(kg c) = cpv - cw
                !
                tc     = tsp(i,k) - t0
                lflg   = (tc >= -ttrice .AND. tc < 0.0_r8)
                weight(i) = MIN(-tc*trinv,1.0_r8)
                hlatsb = hlatv + weight(i)*hlatf
                hlatvp = hlatv - 2369.0_r8*tc
                IF (tsp(i,k) < t0) THEN
                   hltalt(i,k) = hlatsb
                ELSE
                   hltalt(i,k) = hlatvp
                END IF
                IF (lflg) THEN
                   tterm = pcf(1) + tc*(pcf(2) + tc*(pcf(3)+tc*(pcf(4) + tc*pcf(5))))
                ELSE
                   tterm = 0.0_r8
                END IF
                desdt = hltalt(i,k)*es/(rgasv*tsp(i,k)*tsp(i,k)) + tterm*trinv
                dqsdt = (epsqs + omeps*qs)/(p(i,k) - omeps*es)*desdt
                !              g = cp*(tlim(i)-tsp(i,k)) + hltalt(i,k)*q(i,k)- hltalt(i,k)*qsp(i,k)
                g = enin(i) - (cp*tsp(i,k) + hltalt(i,k)*qsp(i,k))
                dgdt = -(cp + hltalt(i,k)*dqsdt)
                t1 = tsp(i,k) - g/dgdt
                dt = ABS(t1 - tsp(i,k))/t1
                tsp(i,k) = MAX(t1,tmin)
                es = estblf(tsp(i,k))
                q1 = MIN(epsqs*es/(p(i,k) - omeps*es),1.0_r8)
                q1=MAX(q1,1.e-12_r8)
                dq = ABS(q1 - qsp(i,k))/MAX(q1,1.e-12_r8)
                qsp(i,k) = q1
                !#ifdef DEBUG
                !               if ( (lchnk == lchnklook(nlook) ) .and. (i == icollook(nlook) ) ) then
                !                  write (6,*) ' rel chg lev, iter, t, q ', k, l, dt, dq, g
                !               endif
                !#endif
                dtm = MAX(dtm,dt)
                dqm = MAX(dqm,dq)
                ! if converged at this point, exclude it from more iterations
                IF (dt < dttol .AND. dq < dqtol) THEN
                   doit(i) = 2
                ENDIF
                enout(i) = cp*tsp(i,k) + hltalt(i,k)*qsp(i,k)
                ! bail out if we are too near the end of temp range
                IF (tsp(i,k) < 174.16_r8) THEN
                   doit(i) = 4
                ENDIF
             ELSE
             ENDIF
          END DO              ! do i = 1,ncols

          IF (dtm < dttol .AND. dqm < dqtol) THEN
             go to 10
          ENDIF

       END DO                 ! do l = 1,iter
10     CONTINUE

       error_found = .FALSE.
       IF (dtm > dttol .OR. dqm > dqtol) THEN
          DO i = 1,ncols
             IF (doit(i) == 0) error_found = .TRUE.
          END DO
          IF (error_found) THEN
             DO i = 1,ncols
                IF (doit(i) == 0) THEN
                   WRITE (6,*) ' findsp not converging at point i, k ', i, k
                   WRITE (6,*) ' t, q, p, enin ', t(i,k), q(i,k), p(i,k), enin(i)
                   WRITE (6,*) ' tsp, qsp, enout ', tsp(i,k), qsp(i,k), enout(i)
                   STOP 'FINDSP'
                ENDIF
             END DO
          ENDIF
       ENDIF
       DO i = 1,ncols
          IF (doit(i) == 2 .AND. ABS((enin(i)-enout(i))/(enin(i)+enout(i))) > 1.e-4_r8) THEN
             error_found = .TRUE.
          ENDIF
       END DO
       IF (error_found) THEN
          DO i = 1,ncols
             IF (doit(i) == 2 .AND. ABS((enin(i)-enout(i))/(enin(i)+enout(i))) > 1.e-4_r8) THEN
                WRITE (6,*) ' the enthalpy is not conserved for point ', &
                     i, k, enin(i), enout(i)
                WRITE (6,*) ' t, q, p, enin ', t(i,k), q(i,k), p(i,k), enin(i)
                WRITE (6,*) ' tsp, qsp, enout ', tsp(i,k), qsp(i,k), enout(i)
                STOP 'FINDSP'
             ENDIF
          END DO
       ENDIF

    END DO                    ! level loop (k=1,pver)

    RETURN
  END SUBROUTINE findsp

  SUBROUTINE findsp_mask (istart,iend,nCols,pver, q, t, p,ierr, tsp, qsp)
    IMPLICIT NONE
    !----------------------------------------------------------------------- 
    ! 
    ! Purpose: 
    !     find the wet bulb temperature for a given t and q
    !     in a longitude height section
    !     wet bulb temp is the temperature and spec humidity that is 
    !     just saturated and has the same enthalpy
    !     if q > qs(t) then tsp > t and qsp = qs(tsp) < q
    !     if q < qs(t) then tsp < t and qsp = qs(tsp) > q
    !
    ! Method: 
    ! a Newton method is used
    ! first guess uses an algorithm provided by John Petch from the UKMO
    ! we exclude points where the physical situation is unrealistic
    ! e.g. where the temperature is outside the range of validity for the
    !      saturation vapor pressure, or where the water vapor pressure
    !      exceeds the ambient pressure, or the saturation specific humidity is 
    !      unrealistic
    ! 
    ! Author: P. Rasch
    ! 
    !-----------------------------------------------------------------------
    !
    !     input arguments
    !

    INTEGER, INTENT(in) :: nCols                 ! number of columns (max)
    INTEGER, INTENT(in) :: pver                  ! number of vertical levels
    INTEGER, INTENT(IN   )    :: istart
    INTEGER, INTENT(IN   )    :: iend
    REAL(KIND=r8), INTENT(in) :: q(nCols,pver)        ! water vapor (kg/kg)
    REAL(KIND=r8), INTENT(in) :: t(nCols,pver)        ! temperature (K)
    REAL(KIND=r8), INTENT(in) :: p(nCols,pver)        ! pressure    (Pa)
    INTEGER, INTENT(IN   )    :: ierr(nCols)
    !
    ! output arguments
    !
    REAL(KIND=r8), INTENT(out) :: tsp(nCols,pver)      ! saturation temp (K)
    REAL(KIND=r8), INTENT(out) :: qsp(nCols,pver)      ! saturation mixing ratio (kg/kg)
    !
    ! local variables
    !
    INTEGER i                 ! work variable
    INTEGER k                 ! work variable
    LOGICAL lflg              ! work variable
    INTEGER iter              ! work variable
    INTEGER l                 ! work variable
    LOGICAL :: error_found

    REAL(KIND=r8) omeps                ! 1 minus epsilon
    REAL(KIND=r8) trinv                ! work variable
    REAL(KIND=r8) es                   ! sat. vapor pressure
    REAL(KIND=r8) desdt                ! change in sat vap pressure wrt temperature
    !     real(KIND=r8) desdp                ! change in sat vap pressure wrt pressure
    REAL(KIND=r8) dqsdt                ! change in sat spec. hum. wrt temperature
    REAL(KIND=r8) dgdt                 ! work variable
    REAL(KIND=r8) g                    ! work variable
    REAL(KIND=r8) weight(nCols)        ! work variable
    REAL(KIND=r8) hlatsb               ! (sublimation)
    REAL(KIND=r8) hlatvp               ! (vaporization)
    REAL(KIND=r8) hltalt(nCols,pver)   ! lat. heat. of vap.
    REAL(KIND=r8) tterm                ! work var.
    REAL(KIND=r8) qs                   ! spec. hum. of water vapor
    REAL(KIND=r8) tc                   ! crit temp of transition to ice

    ! work variables
    REAL(KIND=r8) t1, q1, dt, dq
    REAL(KIND=r8) dtm, dqm
    REAL(KIND=r8) qvd, a1, tmp
    REAL(KIND=r8) rair
    REAL(KIND=r8) r1b, c1, c2, c3
    REAL(KIND=r8) denom
    REAL(KIND=r8) dttol
    REAL(KIND=r8) dqtol
    INTEGER doit(nCols) 
    REAL(KIND=r8) enin(nCols), enout(nCols)
    REAL(KIND=r8) tlim(nCols)
    REAL(KIND=r8) epsqs
    epsqs = epsilo
    k1mb=1
    omeps = 1.0_r8 - epsqs
    trinv = 1.0_r8/ttrice
    a1 = 7.5_r8*LOG(10.0_r8)
    rair =  287.04_r8
    c3 = rair*a1/cp
    dtm = 0.0_r8    ! needed for iter=0 blowup with f90 -ei
    dqm = 0.0_r8    ! needed for iter=0 blowup with f90 -ei
    dttol = 1.e-4_r8 ! the relative temp error tolerance required to quit the iteration
    dqtol = 1.e-4_r8 ! the relative moisture error tolerance required to quit the iteration
    !  tmin = 173.16_r8 ! the coldest temperature we can deal with
    !
    ! max number of times to iterate the calculation
    iter = 50
    !
    DO k = k1mb,pver

       !
       ! first guess on the wet bulb temperature
       !
       DO i =istart,iend 
       IF(ierr(i) == 0)THEN    
          ! limit the temperature range to that relevant to the sat vap pres tables

          tlim(i) = MIN(MAX(t(i,k),173.0_r8),373.0_r8)

          es = estblf(tlim(i))

          denom = p(i,k) - omeps*es
          qs = epsqs*es/denom
          doit(i) = 0
          enout(i) = 1.0_r8
          ! make sure a meaningful calculation is possible
          IF (p(i,k) > 5.0_r8*es .AND. qs > 0.0_r8 .AND. qs < 0.5_r8) THEN
             !
             ! Saturation specific humidity
             !
             qs = MIN(epsqs*es/denom,1.0_r8)
             !
             ! "generalized" analytic expression for t derivative of es
             !  accurate to within 1 percent for 173.16 < t < 373.16
             !
             ! Weighting of hlat accounts for transition from water to ice
             ! polynomial expression approximates difference between es over
             ! water and es over ice from 0 to -ttrice (C) (min of ttrice is
             ! -40): required for accurate estimate of es derivative in transition
             ! range from ice to water also accounting for change of hlatv with t
             ! above freezing where const slope is given by -2369 j/(kg c) = cpv - cw
             !
             tc     = tlim(i) - t0
             lflg   = (tc >= -ttrice .AND. tc < 0.0_r8)
             weight(i) = MIN(-tc*trinv,1.0_r8)
             hlatsb = hlatv + weight(i)*hlatf
             hlatvp = hlatv - 2369.0_r8*tc
             IF (tlim(i) < t0) THEN
                hltalt(i,k) = hlatsb
             ELSE
                hltalt(i,k) = hlatvp
             END IF
             enin(i) = cp*tlim(i) + hltalt(i,k)*q(i,k)

             ! make a guess at the wet bulb temp using a UKMO algorithm (from J. Petch)
             tmp =  q(i,k) - qs
             c1 = hltalt(i,k)*c3
             c2 = (tlim(i) + 36.0_r8)**2
             r1b    = c2/(c2 + c1*qs)
             qvd   = r1b*tmp
             tsp(i,k) = tlim(i) + ((hltalt(i,k)/cp)*qvd)
             !#ifdef DEBUG
             !             if ( (lchnk == lchnklook(nlook) ) .and. (i == icollook(nlook) ) ) then
             !                write (6,*) ' relative humidity ', q(i,k)/qs
             !                write (6,*) ' first guess ', tsp(i,k)
             !             endif
             !#endif
             es = estblf(tsp(i,k))
             qsp(i,k) = MIN(epsqs*es/(p(i,k) - omeps*es),1.0_r8)
          ELSE
             doit(i) = 1
             tsp(i,k) = tlim(i)
             qsp(i,k) = q(i,k)
             enin(i) = 1.0_r8
          ENDIF
       END IF
       END DO   ! end do i
       !
       ! now iterate on first guess
       !
       DO l = 1, iter
          dtm = 0.0_r8
          dqm = 0.0_r8
          DO i =istart,iend
          IF(ierr(i) == 0)THEN 
             IF (doit(i) == 0) THEN
                es = estblf(tsp(i,k))
                !
                ! Saturation specific humidity
                !
                qs = MIN(epsqs*es/(p(i,k) - omeps*es),1.0_r8)
                !
                ! "generalized" analytic expression for t derivative of es
                ! accurate to within 1 percent for 173.16 < t < 373.16
                !
                ! Weighting of hlat accounts for transition from water to ice
                ! polynomial expression approximates difference between es over
                ! water and es over ice from 0 to -ttrice (C) (min of ttrice is
                ! -40): required for accurate estimate of es derivative in transition
                ! range from ice to water also accounting for change of hlatv with t
                ! above freezing where const slope is given by -2369 j/(kg c) = cpv - cw
                !
                tc     = tsp(i,k) - t0
                lflg   = (tc >= -ttrice .AND. tc < 0.0_r8)
                weight(i) = MIN(-tc*trinv,1.0_r8)
                hlatsb = hlatv + weight(i)*hlatf
                hlatvp = hlatv - 2369.0_r8*tc
                IF (tsp(i,k) < t0) THEN
                   hltalt(i,k) = hlatsb
                ELSE
                   hltalt(i,k) = hlatvp
                END IF
                IF (lflg) THEN
                   tterm = pcf(1) + tc*(pcf(2) + tc*(pcf(3)+tc*(pcf(4) + tc*pcf(5))))
                ELSE
                   tterm = 0.0_r8
                END IF
                desdt = hltalt(i,k)*es/(rgasv*tsp(i,k)*tsp(i,k)) + tterm*trinv
                dqsdt = (epsqs + omeps*qs)/(p(i,k) - omeps*es)*desdt
                !              g = cp*(tlim(i)-tsp(i,k)) + hltalt(i,k)*q(i,k)- hltalt(i,k)*qsp(i,k)
                g = enin(i) - (cp*tsp(i,k) + hltalt(i,k)*qsp(i,k))
                dgdt = -(cp + hltalt(i,k)*dqsdt)
                t1 = tsp(i,k) - g/dgdt
                dt = ABS(t1 - tsp(i,k))/t1
                tsp(i,k) = MAX(t1,tmin)
                es = estblf(tsp(i,k))
                q1 = MIN(epsqs*es/(p(i,k) - omeps*es),1.0_r8)
                q1=MAX(q1,1.e-12_r8)
                dq = ABS(q1 - qsp(i,k))/MAX(q1,1.e-12_r8)
                qsp(i,k) = q1
                !#ifdef DEBUG
                !               if ( (lchnk == lchnklook(nlook) ) .and. (i == icollook(nlook) ) ) then
                !                  write (6,*) ' rel chg lev, iter, t, q ', k, l, dt, dq, g
                !               endif
                !#endif
                dtm = MAX(dtm,dt)
                dqm = MAX(dqm,dq)
                ! if converged at this point, exclude it from more iterations
                IF (dt < dttol .AND. dq < dqtol) THEN
                   doit(i) = 2
                ENDIF
                enout(i) = cp*tsp(i,k) + hltalt(i,k)*qsp(i,k)
                ! bail out if we are too near the end of temp range
                IF (tsp(i,k) < 174.16_r8) THEN
                   doit(i) = 4
                ENDIF
             ELSE
             ENDIF
          END IF
          END DO              ! do i = 1,ncols

          IF (dtm < dttol .AND. dqm < dqtol) THEN
             go to 10
          ENDIF

       END DO                 ! do l = 1,iter
10     CONTINUE

       error_found = .FALSE.
       IF (dtm > dttol .OR. dqm > dqtol) THEN
          DO i =istart,iend
            IF( ierr(i) == 0)THEN
             IF (doit(i) == 0) error_found = .TRUE.
            END IF
          END DO
          IF (error_found) THEN
             DO i =istart,iend
               IF(ierr(i) == 0)THEN
                IF (doit(i) == 0) THEN
                   WRITE (6,*) ' findsp not converging at point i, k ', i, k
                   WRITE (6,*) ' t, q, p, enin ', t(i,k), q(i,k), p(i,k), enin(i)
                   WRITE (6,*) ' tsp, qsp, enout ', tsp(i,k), qsp(i,k), enout(i)
                   STOP 'FINDSP'
                ENDIF
               END IF
             END DO
          ENDIF
       ENDIF
       DO i =istart,iend
        IF(ierr(i) == 0)THEN
          IF (doit(i) == 2 .AND. ABS((enin(i)-enout(i))/(enin(i)+enout(i))) > 1.e-4_r8) THEN
             error_found = .TRUE.
          ENDIF
        END IF
       END DO
       IF (error_found) THEN
          DO i =istart,iend
            IF(ierr(i) == 0)THEN  
             IF (doit(i) == 2 .AND. ABS((enin(i)-enout(i))/(enin(i)+enout(i))) > 1.e-4_r8) THEN
                WRITE (6,*) ' the enthalpy is not conserved for point ', &
                     i, k, enin(i), enout(i)
                WRITE (6,*) ' t, q, p, enin ', t(i,k), q(i,k), p(i,k), enin(i)
                WRITE (6,*) ' tsp, qsp, enout ', tsp(i,k), qsp(i,k), enout(i)
                STOP 'FINDSP'
             ENDIF
             END IF
          END DO
       ENDIF

    END DO                    ! level loop (k=1,pver)

    RETURN
  END SUBROUTINE findsp_mask


  SUBROUTINE gestbl(       )
    !----------------------------------------------------------------------- 
    ! 
    ! Purpose: 
    ! Builds saturation vapor pressure table for later lookup procedure.
    ! 
    ! Method: 
    ! Uses Goff & Gratch (1946) relationships to generate the table
    ! according to a set of free parameters defined below.  Auxiliary
    ! routines are also included for making rapid estimates (well with 1%)
    ! of both es and d(es)/dt for the particular table configuration.
    ! 
    ! Author: J. Hack
    ! 
    !-----------------------------------------------------------------------
    !   use pmgrid, only: masterproc
    !------------------------------Arguments--------------------------------
    !
    ! Input arguments
    !
    !
    !---------------------------Local variables-----------------------------
    !
    REAL(KIND=r8)  epsqs
    REAL(KIND=r8) t             ! Temperature
    INTEGER n          ! Increment counter
    INTEGER lentbl     ! Calculated length of lookup table
    INTEGER itype      ! Ice phase: 0 -> no ice phase
    !            1 -> ice phase, no transitiong
    !           -x -> ice phase, x degree transition
    !
    !-----------------------------------------------------------------------
    !
    ! Set es table parameters
    !
    !   tmin   = tmn       ! Minimum temperature entry in table
    !   tmax   = tmx       ! Maximum temperature entry in table
    !   ttrice = trice     ! Trans. range from es over h2o to es over ice
    !   icephs = ip        ! Ice phase (true or false)
    !
    ! Set physical constants required for es calculation
    !
    epsqs  = epsilo
    !
    lentbl = INT(tmax-tmin+2.000001_r8)
    IF (lentbl .GT. plenest) THEN
       WRITE(6,9000) tmax, tmin, plenest
       STOP 'GESTBL'    ! Abnormal termination
    END IF
    !
    ! Begin building es table.
    ! Check whether ice phase requested.
    ! If so, set appropriate transition range for temperature
    !
    IF (icephs) THEN
       IF (ttrice /= 0.0_r8) THEN
          itype = -ttrice
       ELSE
          itype = 1
       END IF
    ELSE
       itype = 0
    END IF
    !
    !tmn  = 173.16_r8! Minimum temperature entry in table
    !tmx  = 375.16_r8! Maximum temperature entry in table

    t = tmin - 1.0_r8
    DO n=1,lentbl
       t = t + 1.0_r8
       CALL gffgch(t,estbl(n),itype)
    END DO
    !
    DO n=lentbl+1,plenest
       estbl(n) = -99999.0_r8
    END DO
    !
    ! Table complete -- Set coefficients for polynomial approximation of
    ! difference between saturation vapor press over water and saturation
    ! pressure over ice for -ttrice < t < 0 (degrees C). NOTE: polynomial
    ! is valid in the range -40 < t < 0 (degrees C).
    !
    !                  --- Degree 5 approximation ---
    !
    pcf(1) =  5.04469588506e-01_r8
    pcf(2) = -5.47288442819e+00_r8
    pcf(3) = -3.67471858735e-01_r8
    pcf(4) = -8.95963532403e-03_r8
    pcf(5) = -7.78053686625e-05_r8
    !
    !                  --- Degree 6 approximation ---
    !
    !-----pcf(1) =  7.63285250063e-02
    !-----pcf(2) = -5.86048427932e+00
    !-----pcf(3) = -4.38660831780e-01
    !-----pcf(4) = -1.37898276415e-02
    !-----pcf(5) = -2.14444472424e-04
    !-----pcf(6) = -1.36639103771e-06
    !
    !   if (masterproc) then
    !      write(6,*)' *** SATURATION VAPOR PRESSURE TABLE COMPLETED ***'
    !   end if

    RETURN
    !
9000 FORMAT('GESTBL: FATAL ERROR *********************************',/, &
         ' TMAX AND TMIN REQUIRE A LARGER DIMENSION ON THE LENGTH', &
         ' OF THE SATURATION VAPOR PRESSURE TABLE ESTBL(PLENEST)',/, &
         ' TMAX, TMIN, AND PLENEST => ', 2f7.2, i3)
    !
  END SUBROUTINE gestbl
  
  
  REAL(KIND=r8) FUNCTION estblf( td )
    !
    ! Saturation vapor pressure table lookup
    !
    REAL(KIND=r8), INTENT(in) :: td         ! Temperature for saturation lookup
    !
    REAL(KIND=r8) :: e          ! intermediate variable for es look-up
    REAL(KIND=r8) :: tmin       ! min temperature (K) for table
    REAL(KIND=r8) :: tmax       ! max temperature (K) for table

    REAL(KIND=r8) :: ai
    INTEGER  :: i

    tmin=tmn  
    tmax=tmx  

    !
    e = MAX(MIN(td,tmax),tmin)   ! partial pressure
    i = INT(e-tmin)+1
    ai = AINT(e-tmin)
    estblf = (tmin+ai-e+1.0_r8)* &
         estbl(i)-(tmin+ai-e)* &
         estbl(i+1)
  END FUNCTION estblf
  
  
  SUBROUTINE aqsat(t       ,p       ,es      ,qs        ,ii      , &
       ILEN    ,kk      ,kstart  ,kend      )
    !----------------------------------------------------------------------- 
    ! 
    ! Purpose: 
    ! Utility procedure to look up and return saturation vapor pressure from
    ! precomputed table, calculate and return saturation specific humidity
    ! (g/g),for input arrays of temperature and pressure (dimensioned ii,kk)
    ! This routine is useful for evaluating only a selected region in the
    ! vertical.
    ! 
    ! Method: 
    ! <Describe the algorithm(s) used in the routine.> 
    ! <Also include any applicable external references.> 
    ! 
    ! Author: J. Hack
    ! 
    !------------------------------Arguments--------------------------------
    !
    ! Input arguments
    !
    INTEGER  , INTENT(in) :: ii             ! I dimension of arrays t, p, es, qs
    INTEGER  , INTENT(in) :: kk             ! K dimension of arrays t, p, es, qs
    INTEGER  , INTENT(in) :: ILEN           ! Length of vectors in I direction which
    INTEGER  , INTENT(in) :: kstart         ! Starting location in K direction
    INTEGER  , INTENT(in) :: kend           ! Ending location in K direction
    REAL(KIND=r8), INTENT(in) :: t(ii,kk)          ! Temperature [K]
    REAL(KIND=r8), INTENT(in) :: p(ii,kk)          ! Pressure [Pa]
    !
    ! Output arguments
    !
    REAL(KIND=r8), INTENT(out) :: es(ii,kk)         ! Saturation vapor pressure [Pa]
    REAL(KIND=r8), INTENT(out) :: qs(ii,kk)         ! Saturation specific humidity [kg/kg]
    !
    !---------------------------Local workspace-----------------------------
    !
    REAL(KIND=r8) epsqs      ! Ratio of h2o to dry air molecular weights 
    REAL(KIND=r8) omeps             ! 1 - 0.622
    INTEGER i, k           ! Indices
    !
    !-----------------------------------------------------------------------
    !
    epsqs = epsilo
    omeps = 1.0_r8 - epsqs
    DO k=kstart,kend
       DO i=1,ILEN
          es(i,k) = estblf(t(i,k))
          !
          ! Saturation specific humidity
          !
          qs(i,k) = epsqs*es(i,k)/(p(i,k) - omeps*es(i,k))
          !
          ! The following check is to avoid the generation of negative values
          ! that can occur in the upper stratosphere and mesosphere
          !
          qs(i,k) = MIN(1.0_r8,qs(i,k))
          !
          IF (qs(i,k) < 0.0_r8) THEN
             qs(i,k) = 1.0_r8
             es(i,k) = p(i,k)
          END IF
       END DO
    END DO
    !
    RETURN
  END SUBROUTINE aqsat
  SUBROUTINE vqsatd(t       ,p       ,es      ,qs      ,gam      , &
       len     )
    !----------------------------------------------------------------------- 
    ! 
    ! Purpose: 
    ! Utility procedure to look up and return saturation vapor pressure from
    ! precomputed table, calculate and return saturation specific humidity
    ! (g/g), and calculate and return gamma (l/cp)*(d(qsat)/dT).  The same
    ! function as qsatd, but operates on vectors of temperature and pressure
    ! 
    ! Method: 
    ! 
    ! Author: J. Hack
    ! 
    !------------------------------Arguments--------------------------------
    !
    ! Input arguments
    !
    INTEGER, INTENT(in) :: len       ! vector length
    REAL(KIND=r8), INTENT(in) :: t(len)       ! temperature [K]
    REAL(KIND=r8), INTENT(in) :: p(len)       ! pressure [Pa]
    !
    ! Output arguments
    !
    REAL(KIND=r8), INTENT(out) :: es(len)   ! saturation vapor pressure
    REAL(KIND=r8), INTENT(out) :: qs(len)   ! saturation specific humidity
    REAL(KIND=r8), INTENT(out) :: gam(len)  ! (l/cp)*(d(qs)/dt)
    !
    !--------------------------Local Variables------------------------------
    !
    LOGICAL lflg        ! true if in temperature transition region
    !
    INTEGER i           ! index for vector calculations
    !
    REAL(KIND=r8) omeps     ! 1. - 0.622
    REAL(KIND=r8) trinv     ! reciprocal of ttrice (transition range)
    REAL(KIND=r8) tc        ! temperature (in degrees C)
    REAL(KIND=r8) weight    ! weight for es transition from water to ice
    REAL(KIND=r8) hltalt    ! appropriately modified hlat for T derivatives
    !
    REAL(KIND=r8) hlatsb    ! hlat weighted in transition region
    REAL(KIND=r8) hlatvp    ! hlat modified for t changes above freezing
    REAL(KIND=r8) tterm     ! account for d(es)/dT in transition region
    REAL(KIND=r8) desdt     ! d(es)/dT
    REAL(KIND=r8) epsqs
    !
    !-----------------------------------------------------------------------
    !
    epsqs = epsilo

    omeps = 1.0_r8 - epsqs
    DO i=1,len
       es(i) = estblf(t(i))
       !
       ! Saturation specific humidity
       !
       qs(i) = epsqs*es(i)/(p(i) - omeps*es(i))
       !
       ! The following check is to avoid the generation of negative
       ! values that can occur in the upper stratosphere and mesosphere
       !
       qs(i) = MIN(1.0_r8,qs(i))
       !
       IF (qs(i) < 0.0_r8) THEN
          qs(i) = 1.0_r8
          es(i) = p(i)
       END IF
    END DO
    !
    ! "generalized" analytic expression for t derivative of es
    ! accurate to within 1 percent for 173.16 < t < 373.16
    !
    trinv = 0.0_r8
    IF ((.NOT. icephs) .OR. (ttrice.EQ.0.0_r8)) go to 10
    trinv = 1.0_r8/ttrice
    DO i=1,len
       !
       ! Weighting of hlat accounts for transition from water to ice
       ! polynomial expression approximates difference between es over
       ! water and es over ice from 0 to -ttrice (C) (min of ttrice is
       ! -40): required for accurate estimate of es derivative in transition
       ! range from ice to water also accounting for change of hlatv with t
       ! above freezing where const slope is given by -2369 j/(kg c) = cpv - cw
       !
       tc     = t(i) - tmelt
       lflg   = (tc >= -ttrice .AND. tc < 0.0_r8)
       weight = MIN(-tc*trinv,1.0_r8)
       hlatsb = hlatv + weight*hlatf
       hlatvp = hlatv - 2369.0_r8*tc
       IF (t(i) < tmelt) THEN
          hltalt = hlatsb
       ELSE
          hltalt = hlatvp
       END IF
       IF (lflg) THEN
          tterm = pcf(1) + tc*(pcf(2) + tc*(pcf(3) + tc*(pcf(4) + tc*pcf(5))))
       ELSE
          tterm = 0.0_r8
       END IF
       desdt  = hltalt*es(i)/(rgasv*t(i)*t(i)) + tterm*trinv
       gam(i) = hltalt*qs(i)*p(i)*desdt/(cp*es(i)*(p(i) - omeps*es(i)))
       IF (qs(i) == 1.0_r8) gam(i) = 0.0_r8
    END DO
    RETURN
    !
    ! No icephs or water to ice transition
    !
10  DO i=1,len
       !
       ! Account for change of hlatv with t above freezing where
       ! constant slope is given by -2369 j/(kg c) = cpv - cw
       !
       hlatvp = hlatv - 2369.0_r8*(t(i)-tmelt)
       IF (icephs) THEN
          hlatsb = hlatv + hlatf
       ELSE
          hlatsb = hlatv
       END IF
       IF (t(i) < tmelt) THEN
          hltalt = hlatsb
       ELSE
          hltalt = hlatvp
       END IF
       desdt  = hltalt*es(i)/(rgasv*t(i)*t(i))
       gam(i) = hltalt*qs(i)*p(i)*desdt/(cp*es(i)*(p(i) - omeps*es(i)))
       IF (qs(i) == 1.0_r8) gam(i) = 0.0_r8
    END DO
    !
    RETURN
    !
  END SUBROUTINE vqsatd

 SUBROUTINE gffgch(t       ,es      ,itype   )
    !----------------------------------------------------------------------- 
    ! 
    ! Purpose: 
    ! Computes saturation vapor pressure over water and/or over ice using
    ! Goff & Gratch (1946) relationships. 
    ! <Say what the routine does> 
    ! 
    ! Method: 
    ! T (temperature), and itype are input parameters, while es (saturation
    ! vapor pressure) is an output parameter.  The input parameter itype
    ! serves two purposes: a value of zero indicates that saturation vapor
    ! pressures over water are to be returned (regardless of temperature),
    ! while a value of one indicates that saturation vapor pressures over
    ! ice should be returned when t is less than freezing degrees.  If itype
    ! is negative, its absolute value is interpreted to define a temperature
    ! transition region below freezing in which the returned
    ! saturation vapor pressure is a weighted average of the respective ice
    ! and water value.  That is, in the temperature range 0 => -itype
    ! degrees c, the saturation vapor pressures are assumed to be a weighted
    ! average of the vapor pressure over supercooled water and ice (all
    ! water at 0 c; all ice at -itype c).  Maximum transition range => 40 c
    ! 
    ! Author: J. Hack
    ! 
    !-----------------------------------------------------------------------
    !   use shr_kind_mod, only: r8 => shr_kind_r8
    !   use physconst, only: tmelt
    !   use abortutils, only: endrun

    IMPLICIT NONE
    !------------------------------Arguments--------------------------------
    !
    ! Input arguments
    !
    REAL(KIND=r8), INTENT(in) :: t          ! Temperature
    !
    ! Output arguments
    !
    INTEGER, INTENT(inout) :: itype   ! Flag for ice phase and associated transition

    REAL(KIND=r8), INTENT(out) :: es         ! Saturation vapor pressure
    !
    !---------------------------Local variables-----------------------------
    !
    REAL(KIND=r8) e1         ! Intermediate scratch variable for es over water
    REAL(KIND=r8) e2         ! Intermediate scratch variable for es over water
    REAL(KIND=r8) eswtr      ! Saturation vapor pressure over water
    REAL(KIND=r8) f          ! Intermediate scratch variable for es over water
    REAL(KIND=r8) f1         ! Intermediate scratch variable for es over water
    REAL(KIND=r8) f2         ! Intermediate scratch variable for es over water
    REAL(KIND=r8) f3         ! Intermediate scratch variable for es over water
    REAL(KIND=r8) f4         ! Intermediate scratch variable for es over water
    REAL(KIND=r8) f5         ! Intermediate scratch variable for es over water
    REAL(KIND=r8) ps         ! Reference pressure (mb)
    REAL(KIND=r8) t0         ! Reference temperature (freezing point of water)
    REAL(KIND=r8) term1      ! Intermediate scratch variable for es over ice
    REAL(KIND=r8) term2      ! Intermediate scratch variable for es over ice
    REAL(KIND=r8) term3      ! Intermediate scratch variable for es over ice
    REAL(KIND=r8) tr         ! Transition range for es over water to es over ice
    REAL(KIND=r8) ts         ! Reference temperature (boiling point of water)
    REAL(KIND=r8) weight     ! Intermediate scratch variable for es transition
    INTEGER itypo   ! Intermediate scratch variable for holding itype
    !
    !-----------------------------------------------------------------------
    !
    ! Check on whether there is to be a transition region for es
    !
    IF (itype < 0) THEN
       tr    = ABS(real(itype,kind=r8))
       itypo = itype
       itype = 1
    ELSE
       tr    = 0.0_r8
       itypo = itype
    END IF
    IF (tr > 40.0_r8) THEN
       WRITE(6,900) tr
       STOP 'GFFGCH'                ! Abnormal termination
    END IF
    !
    IF(t < (tmelt - tr) .AND. itype == 1) go to 10
    !
    ! Water
    !
    ps = 1013.246_r8
    ts = 373.16_r8
    e1 = 11.344_r8*(1.0_r8 - t/ts)
    e2 = -3.49149_r8*(ts/t - 1.0_r8)
    f1 = -7.90298_r8*(ts/t - 1.0_r8)
    f2 = 5.02808_r8*LOG10(ts/t)
    f3 = -1.3816_r8*(10.0_r8**e1 - 1.0_r8)/10000000.0_r8
    f4 = 8.1328_r8*(10.0_r8**e2 - 1.0_r8)/1000.0_r8
    f5 = LOG10(ps)
    f  = f1 + f2 + f3 + f4 + f5
    es = (10.0_r8**f)*100.0_r8
    eswtr = es
    !
    IF(t >= tmelt .OR. itype == 0) go to 20
    !
    ! Ice
    !
10  CONTINUE
    t0    = tmelt
    term1 = 2.01889049_r8/(t0/t)
    term2 = 3.56654_r8*LOG(t0/t)
    term3 = 20.947031_r8*(t0/t)
    es    = 575.185606e10_r8*EXP(-(term1 + term2 + term3))
    !
    IF (t < (tmelt - tr)) go to 20
    !
    ! Weighted transition between water and ice
    !
    weight = MIN((tmelt - t)/tr,1.0_r8)
    es = weight*es + (1.0_r8 - weight)*eswtr
    !
20  CONTINUE
    itype = itypo
    RETURN
    !
900 FORMAT('GFFGCH: FATAL ERROR ******************************',/, &
         'TRANSITION RANGE FOR WATER TO ICE SATURATION VAPOR', &
         ' PRESSURE, TR, EXCEEDS MAXIMUM ALLOWABLE VALUE OF', &
         ' 40.0 DEGREES C',/, ' TR = ',f7.2)
    !
  END SUBROUTINE gffgch
END MODULE wv_saturation


 MODULE CloudFractionModel
  USE wv_saturation, Only :findsp
  IMPLICIT NONE
  PRIVATE
  INTEGER, PARAMETER, PUBLIC :: r4 = SELECTED_REAL_KIND(6)  ! Kind for 32-bits Real Numbers
  INTEGER, PARAMETER, PUBLIC :: r8 = SELECTED_REAL_KIND(15) ! Kind for 64-bits Real Numbers
  REAL (KIND=r8), PARAMETER   :: cp    =                  1004.6_r8! specific heat of air           (j/kg/k)
  REAL (KIND=r8), PARAMETER   :: gasr  =                  287.05_r8! gas constant of dry air        (j/kg/k)
  CHARACTER(LEN=*),PARAMETER :: iccon='ARA'
  
  
  
  PUBLIC :: cldgn3
  PUBLIC :: CloudFractionLayrson
  CONTAINS
  
  !   This version is intended for, and was tested with, the new
  !shortwave radiation codes: clirad and ukmet
  !
  SUBROUTINE cldgn3 (prsl  , &!REAL(KIND=r8),INTENT(in   ) :: prsl  (ncols,kMax)   ==> pressure   (Pa)
                     prsi  , &!REAL(KIND=r8),INTENT(in   ) :: prsi  (ncols,kMax+1) ==> interface layer pressure   (Pa)
                     shmt  , &!REAL(KIND=r8),INTENT(in   ) :: grh   (ncols,kmax)   ==> specific humidity  (kg/kg)
                     gice  , &!REAL(KIND=r8),INTENT(in   ) :: grh   (ncols,kmax)   ==> ice concentration  (kg/kg)
                     gliq  , &!REAL(KIND=r8),INTENT(in   ) :: grh   (ncols,kmax)   ==> liquid concentration  (kg/kg)
                     grh   , &!REAL(KIND=r8),INTENT(in   ) :: grh   (ncols,kmax)   ==> relative humidity  (0-1)
                     omg   , &!REAL(KIND=r8),INTENT(in   ) :: omg   (ncols,kmax)   ==> vertical velocity  (cb/sec)
                     gtmp  , &!REAL(KIND=r8),INTENT(in   ) :: gtmp  (ncols,kmax)   ==> layer temperature (k)
                     prec  , &!REAL(KIND=r8),INTENT(in   ) :: prcpt (ncols)        ==> precipitation at each time step.(m)
                     convc , &!REAL(KIND=r8),INTENT(in   ) :: convc (ncols)        ==> convective cloud cover in 3 hr. avrage
                     convt , &!REAL(KIND=r8),INTENT(in   ) :: convt (ncols)        ==> ncols convective cloud top  (sigma layer)
                     convb , &!REAL(KIND=r8),INTENT(in   ) :: convb (ncols)        ==> ncols convective cloud base (sigma layer)
                     css   , &!REAL(KIND=r8),INTENT(inout) :: css   (ncols,kmax)   ==> supersatuation cloud cover fraction
                     ccu   , &!REAL(KIND=r8),INTENT(inout) :: ccu   (ncols,kmax)   ==> convective cloud cover fraction
                     cdin  , &!REAL(KIND=r8),INTENT(inout) :: cdin  (ncols,kmax)   ==> Inversion clouds
                     cstc  , &!REAL(KIND=r8),INTENT(inout) :: cstc  (ncols,kmax)   ==> Saturation clouds
                     ccon  , &!REAL(KIND=r8),INTENT(inout) :: ccon  (ncols,kmax)   ==> Convection clouds 
                     cson  , &!REAL(KIND=r8),INTENT(inout) :: cson  (ncols,kmax)   ==> Shallow convective clouds
                     auxl  , &!REAL(KIND=r8),INTENT(inout) :: auxl  (ncols,kmax)   ==> clouds
                     lcnvl , &!INTEGER      ,INTENT(in   ) :: lcnvl       ==>  the lowest layer index where non-convective clouds can occur
                     lthncl, &!INTEGER      ,INTENT(in   ) :: lthncl      ==>  Minimum depth in mb of non-zero low level cloud constant
                     ncols , &!INTEGER      ,INTENT(IN   ) :: ncols       ==>  Number of colum
                     kmax  , &!INTEGER      ,INTENT(in   ) :: kmax        ==>  Number of layer
                     nls     )!INTEGER      ,INTENT(in   ) :: nls         ==>  Number of layers in the stratosphere.

    !==========================================================================
    ! cldgen :perform clouds generation scheme based on j. slingo
    !         (1984 ecmwf workshop); the scheme generates 4 type of clouds
    !         of convective, high, middle and low clouds.
    !==========================================================================
    ! parameters and input variables:
    !        covlp = 'maxi'         maximum overlap of convective cloud
    !                               or thick low cloud used in ir subr.
    !              = 'rand'         random  overlap of convective cloud
    !                               or thick low cloud used in ir subr.
    !        date  =  julian day of model forecast date
    !        grh   =  relative humidity  (0-1)
    !        omg   =  vertical velocity  (cb/sec)
    !        prsl  =  pressure   (Pa)
    !        sigmid   =  sigma coordinate at middle of layer
    !        gtmp  =  layer temperature (k)
    !     output variables:
    !        css   =  ncols*kmax supersatuation cloud cover fraction
    !        ccu   =  ncols*kmax convective cloud cover fraction
    !---------------------------------------------------------------------
    ! values from subr-gwater
    !       convc  =  ncols convective cloud cover in 3 hr. avrage
    !       convt  =  ncols convective cloud top  (sigma layer)
    !       convb  =  ncols convective cloud base (sigma layer)
    !==========================================================================
    !
    !   ncols.....Number of grid points on a gaussian latitude circle
    !   kmax......Number of grid points at vertical
    !   nls..... .Number of layers in the stratosphere.
    !   cdin......Inversion clouds
    !   cstc......cstc=clow change necessary in order to properly mark inv
    !             cloud height
    !   ccon......convc  =  ncols convective cloud cover in 3 hr. avrage
    !   cson......Shallow convective clouds
    !   cp........Specific heat of air (j/kg/k)
    !   gasr......Constant of dry air      (j/kg/k)
    !   mxrdcc....use maximum random converage for radiative conv. clouds
    !             constant logical mxrdcc = .true.
    !   lcnvl.....the lowest layer index where non-convective clouds can
    !             occur (ben says this should be 2 or more)
    !             constant lcnvl = 2
    !   lthncl....Minimum depth in mb of non-zero low level cloud
    !             consta lthncl=80
    !==========================================================================
    INTEGER      ,             INTENT(IN   ) :: ncols
    INTEGER      ,             INTENT(in   ) :: kmax
    INTEGER      ,             INTENT(in   ) :: nls
    REAL(KIND=r8),             INTENT(in   ) :: prsl  (ncols,kMax)
    REAL(KIND=r8),             INTENT(in   ) :: prsi  (ncols,kMax+1)
    REAL(KIND=r8),             INTENT(in   ) :: shmt  (ncols,kmax)
    REAL(KIND=r8),             INTENT(in   ) :: gice  (ncols,kmax)
    REAL(KIND=r8),             INTENT(in   ) :: gliq  (ncols,kmax)
    REAL(KIND=r8),             INTENT(in   ) :: grh   (ncols,kmax)
    REAL(KIND=r8),             INTENT(in   ) :: omg   (ncols,kmax)
    REAL(KIND=r8),             INTENT(in   ) :: gtmp  (ncols,kmax)
    REAL(KIND=r8),             INTENT(in   ) :: prec (ncols)
    REAL(KIND=r8),             INTENT(inout) :: css   (ncols,kmax)
    REAL(KIND=r8),             INTENT(inout) :: ccu   (ncols,kmax)
    REAL(KIND=r8),             INTENT(inout) :: cdin  (ncols,kmax)
    REAL(KIND=r8),             INTENT(inout) :: cstc  (ncols,kmax)
    REAL(KIND=r8),             INTENT(inout) :: ccon  (ncols,kmax)
    REAL(KIND=r8),             INTENT(inout) :: cson  (ncols,kmax)
    REAL(KIND=r8),             INTENT(inout) :: auxl   (ncols,kmax)
    INTEGER      ,             INTENT(in   ) :: lcnvl
    INTEGER      ,             INTENT(in   ) :: lthncl
    REAL(KIND=r8),             INTENT(inout) :: convc (ncols)
    REAL(KIND=r8),             INTENT(inout) :: convt (ncols)
    REAL(KIND=r8),             INTENT(inout) :: convb (ncols)
    !
    !     dthdpm-----min (d(theta)/d(p))
    !     invb ------the base of inversion layer
    !     dthdpc-----criterion of (d(theta)/d(p))
    !
    REAL(KIND=r8),    PARAMETER :: dthdpc = -0.4e-1_r8! dthdpc= -0.4e-1_r8-----criterion of (d(theta)/d(p))
    REAL(KIND=r8)    :: pll   (ncols,kmax)
    REAL(KIND=r8)    :: pii   (ncols,kmax+1)
    REAL(KIND=r8)    :: cauxil1(ncols,kmax)
    REAL(KIND=r8)    :: cauxil2(ncols,kmax)
    REAL(KIND=r8)     :: prcpt (ncols)

    REAL(KIND=r8)    :: conv  (ncols)
    REAL(KIND=r8)    :: clow  (ncols)
    REAL(KIND=r8)    :: cmid  (ncols)
    REAL(KIND=r8)    :: chigh (ncols)! high clouds due to strong convection
    INTEGER          :: ktop  (ncols)
    INTEGER          :: kbot  (ncols)
    INTEGER          :: klow  (ncols)
    INTEGER          :: kmid  (ncols)
    INTEGER          :: khigh (ncols)! index of the high clouds due to strong convection
    REAL(KIND=r8)    :: pt    (ncols)
    REAL(KIND=r8)    :: cx    (ncols)
    REAL(KIND=r8)    :: cinv  (ncols)!low stratus associated with inversions
    REAL(KIND=r8)    :: omeganeg(ncols,kmax)
    REAL(KIND=r8)    :: delomg(ncols)
    REAL(KIND=r8)    :: dthdpm(ncols)
    INTEGER          :: invb  (ncols)
    REAL(KIND=r8)    :: SummOmeg(ncols)
    REAL(KIND=r8)    :: CounOmeg(ncols)
    REAL(KIND=r8)    :: AverOmeg(ncols)
    REAL(KIND=r8)    :: VariOmeg(ncols)
    REAL(KIND=r8)    :: normal(ncols,kmax)
    REAL(KIND=r8)    :: CLDF_PDF (ncols,kmax)

    REAL(KIND=r8), PARAMETER :: f700p= 8.0e2_r8!7.5e2_r8! 7.0e2_r8
    REAL(KIND=r8), PARAMETER :: f400p= 4.0e2_r8
    REAL(KIND=r8), PARAMETER :: f6p67= 6.67e0_r8
    REAL(KIND=r8), PARAMETER :: f0p9 = 0.9e0_r8!Critical Relative Humidity   up level 400mb
    REAL(KIND=r8), PARAMETER :: f0p8 = 0.8e0_r8!Critical Relative Humidity down level 400mb 
    REAL(KIND=r8), PARAMETER :: f0p4 = 0.4e0_r8!Critical cloud due precipitation
    REAL(KIND=r8), PARAMETER :: f0p3 = 0.3e0_r8
    REAL(KIND=r8), PARAMETER :: f0p2 = 0.2e0_r8
    REAL(KIND=r8), PARAMETER :: f0p1 = 0.1e0_r8
    REAL(KIND=r8), PARAMETER :: f0p6 = 0.6e0_r8!Critical Relative Humidity to  stratus cloud
    REAL(KIND=r8), PARAMETER :: f1e4 = 1.0e+4_r8
    REAL(KIND=r8), PARAMETER :: f5m5 = 5.0e-5_r8!Critical vertical Velocity cb/s
    REAL(KIND=r8), PARAMETER :: f1m5 = 5.0e-5_r8!Critical vertical Velocity cb/s

    INTEGER :: i
    INTEGER :: k
    INTEGER :: kl
    INTEGER :: lon
    REAL(KIND=r8)    :: arcp
    REAL(KIND=r8)    :: dthdp
    REAL(KIND=r8)    :: thklow
    REAL(KIND=r8)    :: maxomeg

    REAL(KIND=r8)   , PARAMETER :: fp2457 = 0.2457_r8
    REAL(KIND=r8)   , PARAMETER :: fp1253 = 0.1253_r8
    REAL(KIND=r8)   , PARAMETER :: f8p0e3 = 8.0e3_r8  ! 1000 * 8

    DO i = 1, ncols
       pii   (i,kmax+1)= 0.0_r8
       conv  (i)= 0.0_r8
       clow  (i)= 0.0_r8
       cmid  (i)= 0.0_r8
       chigh (i)= 0.0_r8
       ktop  (i)= 0.0_r8
       kbot  (i)= 0.0_r8
       klow  (i)= 0.0_r8
       kmid  (i)= 0.0_r8
       khigh (i)= 0.0_r8
       pt    (i)= 0.0_r8
       cx    (i)= 0.0_r8
       cinv  (i)= 0.0_r8
       delomg(i)= 0.0_r8
       dthdpm(i)= 0.0_r8
       invb  (i)= 0.0_r8
       SummOmeg(i)= 0.0_r8
       CounOmeg(i)= 0.0_r8
       AverOmeg(i)= 0.0_r8
       VariOmeg(i)= 0.0_r8
       convc(i)= 0.0_r8
    END DO

    DO k = 1, kmax
       DO i = 1, ncols
          auxl(i,k)= 0.0_r8
          pii (i,k)= 0.0_r8
          pll (i,k) = 0.0_r8
          cauxil1(i,k) = 0.0_r8
          cauxil2(i,k) = 0.0_r8
          css(i,k)  = 0.0_r8
          ccu(i,k)  = 0.0_r8
          cdin(i,k) = 0.0_r8
          cstc(i,k) = 0.0_r8
          ccon(i,k) = 0.0_r8
          cson(i,k) = 0.0_r8
          normal(i,k) = 0.0_r8
          CLDF_PDF(i,k) = 0.0_r8
       END DO
    END DO
   
    CALL CLOUDS_GNO(ncols,kMax,prsl,gtmp,shmt,gice,gliq,CLDF_PDF)

    !--------------------------------------------------------
    !   CLOUD COVER, Cloud TOP-BOT FOR RADIATION (sub cldgen)
    !             DUE CONV PRECIPITATION
    !--------------------------------------------------------
    !   TO cldgen are necessary:
    !   a) cloud top and base   (convt, convb in cldgen)
    !   b) cloud amount is calculated convc (only rrr>0). It is calculate below.
    !     a+b used to defined high clouds due to strong convection
    !   prcpt=precipitation at each time step.(rrr) meters
    !   convt=ktop
    !   conbt=kbot (>=2) for radiation
    !*****************************************************************
    !   DO i = 1, ncols
    !      prcpt(i) = prcpt(i) - prcp1(i) + prcp3(i)!m
    !   END DO
    DO i = 1, ncols 
       IF(prec(i) <0.02_r8)then
          prcpt(i) =0.0e0_r8
       ELSE
          prcpt(i) =prec(i)
       END IF
    END DO

    IF(TRIM(iccon).EQ.'ARA')THEN
      DO i = 1, ncols 
         IF (prcpt(i) .GT. 0.0e0_r8) THEN
            convc(i) = fp2457 + fp1253 * LOG(prcpt(i) * f8p0e3)
            convc(i) = MAX(convc(i), 0.0e0_r8)
            convc(i) = MIN(convc(i), f0p8)
         END IF
      END DO
    ELSE
      DO i = 1, ncols 
         IF (prcpt(i) .GT. 0.0e0_r8) THEN
            convc(i) = fp2457 + fp1253 * LOG(prcpt(i) * f8p0e3)
            convc(i) = MAX(convc(i), 0.0e0_r8)
            convc(i) = MIN(convc(i), f0p8)
         END IF
      END DO
    END IF  

    DO kl = 1, kmax
       DO lon = 1, ncols 
          IF(omg(lon,kl) < 0.0e0_r8 ) THEN
             omeganeg(lon,kl)=omg(lon,kl)
          ELSE
             omeganeg(lon,kl)=0.0e0_r8
          END IF  
       END DO        
    END DO
    !
    !     the clouds generation scheme is based on j. slingo
    !     (1984 ecmwf workshop).  the scheme generates 4 type of clouds
    !     of convective, high, middle and low clouds.
    !
    DO kl = 1, kmax
       DO lon = 1,ncols
          pll(lon,kl) = prsl(lon,kl)/100.0_r8!gps(lon) * sigmid(kl)
       END DO
    END DO

    DO kl = 1, kmax+1
       DO lon = 1,ncols
          pii(lon,kl) = prsi(lon,kl)/100.0_r8!gps(lon) * sigmid(kl)
       END DO
    END DO
    DO kl = 1, kmax
       DO lon = 1,ncols
          SummOmeg(lon) =SummOmeg(lon)+omeganeg(lon,kl)
          CounOmeg(lon) = CounOmeg(lon)+1.0_r8
       END DO
    END DO
    DO lon = 1,ncols
       AverOmeg(lon) = SummOmeg(lon)/CounOmeg(lon)
    END DO
    DO kl = 1, kmax
       DO lon = 1,ncols
          VariOmeg(lon) = VariOmeg(lon) + ((omeganeg(lon,kl)-AverOmeg(lon))**2)
       END DO
    END DO
    DO lon = 1,ncols
       VariOmeg(lon) =SQRT(VariOmeg(lon)/CounOmeg(lon))
       IF(VariOmeg(lon) <=0.0)VariOmeg(lon)=0.0000001
    END DO
    DO lon = 1,ncols
        AverOmeg(lon)=MINVAL(omeganeg(lon,1:kmax))
    END DO
    !VariOmeg=VariOmeg!*0.25
    !lambda=2   
    !blognep=2.7182!e => base of the Napierian logarithm
    DO kl = 1, kmax
       DO lon = 1,ncols
          normal(lon,kl) = (1.0_r8/(VariOmeg(lon)*sqrt(2.0_r8*3.14_r8)))* &
          exp(-(1.0_r8/2.0_r8)*((omeganeg(lon,kl)-AverOmeg(lon))/(VariOmeg(lon)))**2)
 !         poisson(lon,kl) = (1.0_r8/(pll(lon,kl)*VariOmeg(lon)*sqrt(2.0_r8*3.14_r8)))* &
 !         exp(-((log(pll(lon,kl))-AverOmeg(lon))**2/(2.0_r8*VariOmeg(lon)*VariOmeg(lon))))
       END DO
    END DO

    maxomeg=MAXVAL(normal)
    DO kl = 1, kmax
       DO lon = 1,ncols
          normal(lon,kl) = normal(lon,kl)/maxomeg
       END DO
    END DO
!    maxomeg=MAXVAL(poisson)
    DO kl = 1, kmax
       DO lon = 1,ncols
         ! poisson(lon,kl) = poisson(lon,kl)/maxomeg
!          PRINT*,'++++++',pll(lon,kl),grh(lon,kl),gtmp(lon,kl),CLDF_PDF(lon,kl) 
       END DO
    END DO
    !
    !     initialization
    !
    DO  lon = 1,ncols
       !
       !clouds due to strong convection
       !
       conv(lon)  = convc(lon)

       clow(lon)  = 0.0_r8
       cinv(lon)  = 0.0_r8
       cmid(lon)  = 0.0_r8
       chigh(lon) = 0.0_r8
       !
       ! Check top cloud
       !       
       ktop(lon)  = INT( convt(lon)+0.5_r8 )
       IF (ktop(lon).LT.1.OR.ktop(lon).GT.kmax) ktop(lon)=1
       !
       ! Check botton cloud
       !       
       kbot(lon)  = INT(convb(lon)+0.5_r8)
       IF (kbot(lon).LT.1.OR.kbot(lon).GT.kmax) kbot(lon)=1
       !
       klow(lon)  = 1
       kmid(lon)  = 1
       khigh(lon) = 1  !index of the high clouds due to strong convection
       !
       !     1. define convective cloud
       !     done in subr-gwater
       !     cloud top and base are defined by kuo scheme: convt, convb
       !     cloud amount is calculated from precipitation rate : convc
       !     *** single layer clouds conputations start here, from bottom up
       !
       !     define high clouds due to strong convection
       !
       pt(lon) =pll(lon,ktop(lon))
       !
       ! Check the top cloud 
       ! if the top of the cloud is above 400mb and 
       ! the cloud due precipitation  is above 0.4 [Critical cloud due precipitation]
       !
       IF ((  pt(lon).LE.f400p).AND.(conv(lon).GE.f0p4)) THEN
          !
          ! high clouds due to strong convection
          !
          chigh(lon) = 2.0_r8 * (conv(lon) - f0p3)
          chigh(lon) = MIN(chigh(lon),1.0_r8)
          !
          ! index of the high clouds due to strong convection
          !
          khigh(lon) = MIN(ktop(lon) + 1,kmax)
          !
          ! top cloud convection due  strong convection
          !
          ccon(lon,khigh(lon))=chigh(lon)
       END IF
    END DO
    !
    !     compute low stratus associated with inversions, based on ecwmf's
    !     scheme, with lower criterion of d(theta)/d(p)
    !
    arcp = gasr / cp
    DO lon=1,ncols
       invb  (lon) =  MIN(kmax, kmax-nls)
       dthdpm(lon) = 0.0_r8
    END DO

    DO kl=2,kmax
       DO lon=1,ncols
          !
          ! compute low stratus associated with inversions
          ! Check Levels below 700 mb
          !
          IF (pll(lon,kl) .GT. f700p) THEN
             !
             ! Differency of potential temperature between 2 level [(k-1) - (k)]
             !
             dthdp = (gtmp(lon,kl-1)*(1000.0_r8/pll(lon,kl-1))**arcp &
                     -gtmp(lon,kl  )*(1000.0_r8/pll(lon,kl  ))**arcp)/&
                     (pll(lon,kl-1) - pll(lon,kl))
             !
             !  Conditional inversion
             ! lower criterion of d(theta)/d(p) < 0
             !
             ! theta(k-1)  -  theta(k)      -
             !------------------------- = -----   < 0   ===> stable
             !   P(k-1)    - P(k)           +
             !
             IF(dthdp.LT.0.0_r8) THEN
                !
                !change  index of the inversion layer
                !
                invb(lon) = MIN(kl-1,invb(lon))
                !
                ! dthdpc= -0.4e-1_r8-----criterion of (d(theta)/d(p))
                !
                ! theta(k-1)  -  theta(k)      -
                !------------------------- = -----   < -0.04   ===> near neutral
                !   P(k-1)    - P(k)           +
                !
                !
                IF(dthdp.LT.dthdpc) THEN
                   ! 
                   ! Checks for stability between 2 layers
                   !
                   !    k+1         <      k    
                   !
                   !d(thetap)/d(p)  < d(thetam)/d(p) 
                   !
                   IF(dthdp.LT.dthdpm(lon)) THEN
                      !
                      ! if the stability in the top layer is less than the 
                      ! stability in the bottom layer, the inversion
                      !
                      dthdpm(lon)=dthdp
                      !
                      !  index of the inversion layer
                      !
                      klow(lon)=kl
                   ENDIF
                ENDIF
             ENDIF
          ENDIF
       END DO
    END DO
    !
    !     klow change above necessary to mark inversion cloud height
    !
    DO lon=1,ncols
       !
       ! theta(k-1)  -  theta(k)      -
       !------------------------- = -----   < -0.04   ===> near neutral
       !   P(k-1)    - P(k)           +
       !
       !
       IF(dthdpm(lon).LT.dthdpc .AND. grh(lon,invb(lon)).GT.f0p6)THEN
          !
          ! compute low stratus associated with inversions
          !          below 700 mb
          !
          ! dthdpc= -0.4e-1_r8-----criterion of (d(theta)/d(p))
          !
          cinv(lon) = - f6p67 * (dthdpm(lon)-dthdpc)
          !
          cinv(lon) = MAX(cinv(lon),0.0_r8)
          cinv(lon) = MIN(cinv(lon),1.0_r8)
          !
          ! update low stratus associated with inversions whit relative humidity
          ! if  relative humidity < 0.8
          !
          IF (grh(lon,invb(lon)) .LT. f0p8)THEN
               cinv(lon) = cinv(lon)*(1.0_r8-(f0p8 - grh(lon,invb(lon)))/f0p2)
          END IF
       ENDIF
       clow(lon)=cinv(lon)
       cdin(lon,klow(lon))=cinv(lon)
    END DO
    !
    !     clow change necessary in order to properly mark inv cloud height
    !
    !     main loop for cloud amount determination
    !
    DO kl = lcnvl+1, MIN(kmax, kmax-nls)
       DO lon = 1,ncols
          !
          !     general define cloud due to saturation
          !
          IF (pll(lon,kl) .GT. f400p) THEN
             cx(lon) = (grh(lon,kl) - f0p8) / f0p2
          ELSE
             cx(lon) = (grh(lon,kl) - f0p9) / f0p1
          END IF


          cx(lon) = (MAX(cx(lon), 0.0_r8)) ** 2
          cx(lon) =  MIN(cx(lon), 1.0_r8)
          auxl(lon,kl)=cx(lon)


          !
          !     start vertical process from bottom to top
          !
          IF (pll(lon,kl) .GT. f700p) THEN
             !
             !     2. define low cloud ***
             !     low cloud is defined one layer thick ranging from layer 3 to 700mb
             !     there are two type possible generating mechanisms. due boundary
             !     t inversion type and associated with vertical motion.
             !
             !     define low super satuated clouds but adjusted by vertical motion
             !
             IF (omg(lon,kl) .GE. f5m5) THEN
                !
                ! no satuated cloud  
                !
                cx(lon) = 0.0_r8
                auxl(lon,kl)=cx(lon)
                !
             ELSE IF (omg(lon,kl) .GE. -f5m5) THEN
                !
                ! satuated cloud  
                !
                delomg(lon) = (omg(lon,kl)+f5m5)*f1e4
                !
                cx(lon) = cx(lon) * (1.0_r8 - delomg(lon) * delomg(lon))
                auxl(lon,kl)=cx(lon)
                !
             END IF
             !
             ! update  low cloud due to saturation
             !
             IF (cx(lon) .GT. clow(lon)) THEN
                klow(lon) = kl
                clow(lon) = cx(lon)
             END IF

          ELSE IF (pll(lon,kl) .GT. f400p) THEN
             !
             !     3. define middle cloud ***
             !     middle cloud is defined one layer thick between 700 and 400 mb.
             !
             !     define middle clouds only in supersaturate type
             !
             IF (cx(lon) .GT. cmid(lon)) THEN
                kmid(lon) = kl
                cmid(lon) = cx(lon)
             END IF
          ELSE
             !
             !     4. define high cloud
             !     high cloud is defined only one layer thick from 400 mb and up.
             !
             !     define high clouds due to satuation
             !
             IF (cx(lon) .GT. chigh(lon)) THEN
                khigh(lon) = kl
                chigh(lon) = cx(lon)
             END IF
             !
             !     end of vertical computation
             !
          END IF
       END DO
    END DO



    DO lon = 1,ncols
       css (lon,kmax-khigh(lon)+1) = chigh(lon)
       cstc(lon,khigh(lon))        = chigh(lon)
       css (lon,kmax-kmid(lon)+1)  = cmid(lon)
       cstc(lon,kmid(lon))         = cmid(lon)
       !
       !     for very thin low cloud adding its thickness
       !     pressure thickness of low cloud layer
       !
       IF(klow(lon).GE.lcnvl) THEN
          !
          !pressure thickness of low cloud layer
          !
          thklow=0.5_r8*(pll(lon,klow(lon)-1)-pll(lon,klow(lon)+1))
          !
          !lthncl=80 => Minimum depth in mb of non-zero low level cloud  constant
          !
          IF(thklow.LE.REAL(lthncl,r8)) THEN
             css (lon,kmax-klow(lon)) = clow(lon)
             cstc(lon,klow(lon)+1)     =clow(lon)
          ENDIF
          css(lon,kmax-klow(lon)+1) = clow(lon)
          cstc(lon,klow(lon))       = clow(lon)
       ENDIF
    END DO

    !NEW CODE-------------------------------------------------------------
    DO lon = 1,ncols
       DO kl = kbot(lon), ktop(lon)
          IF(omg   (lon,kl) < -2.0E-004_r8)THEN
             ccu(lon,kmax-kl+1) = MAX(CLDF_PDF(lon,kl),conv(lon)*normal(lon,kl))
             ccon(lon,kl)       = MAX(CLDF_PDF(lon,kl),conv(lon)*normal(lon,kl))
          ELSE
             ccu(lon,kmax-kl+1) = conv(lon)*normal(lon,kl)
             ccon(lon,kl)       = conv(lon)*normal(lon,kl)
          END IF
          css(lon,kmax-kl+1) = MIN(1.0_r8,css(lon,kmax-kl+1))
       END DO
    END DO
    DO lon = 1,ncols
       DO kl = kbot(lon), ktop(lon)
          cauxil1(lon,kl)=css(lon,kl)
          cauxil2(lon,kl)=ccu(lon,kl)
       END DO
    END DO

    DO kl = 1, kmax
       DO lon = 1,ncols
          css(lon,kl)= cauxil1(lon,kmax+1-kl)
          ccu(lon,kl)= cauxil2(lon,kmax+1-kl)
       END DO
    END DO

    RETURN
  END SUBROUTINE cldgn3


  !
  ! $Header$
  !
  !
  !================================================================================
  !
!  SUBROUTINE CLOUDS_GNO(nCols,kMax,R,RS,QSUB,CLDF)
  SUBROUTINE CLOUDS_GNO(nCols,kMax,prsl,t3,q3,gicep,gliqp,CLDF)

    IMPLICIT NONE
    !     
    !--------------------------------------------------------------------------------
    !
    ! Inputs:
    !
    !  kMax----------: Number of vertical levels
    !  R--------kMax-: Domain-averaged mixing ratio of total water 
    !  RS-------kMax-: Mean saturation humidity mixing ratio within the gridbox
    !  QSUB-----kMax-: Mixing ratio of condensed water within clouds associated
    !                with SUBGRID-SCALE condensation processes (here, it is
    !                predicted by the convection scheme)
    ! Outputs:
    !
    !  PTCONV-----kMax-: Point convectif = TRUE
    !  RATQSC-----kMax-: Normalized width of the distribution
    !  CLDF-----kMax-: Fraction nuageuse
    !
    !--------------------------------------------------------------------------------


    INTEGER      , INTENT(IN   ) :: nCols
    INTEGER      , INTENT(IN   ) :: kMax
    REAL(KIND=r8), INTENT(IN   ) :: prsl(nCols,kMax)
    REAL(KIND=r8), INTENT(IN   ) :: t3(nCols,kMax)
    REAL(KIND=r8), INTENT(IN   ) :: q3(nCols,kMax)
    REAL(KIND=r8), INTENT(IN   ) :: gicep(nCols,kMax)
    REAL(KIND=r8), INTENT(IN   ) :: gliqp(nCols,kMax)
    REAL(KIND=r8), INTENT(OUT  ) :: CLDF(nCols,kMax)
    ! -- parameters controlling the iteration:
    ! --    nmax    : maximum nb of iterations (hopefully never reached)
    ! --    epsilon : accuracy of the numerical resolution 
    ! --    Omega    : v-value above which we use an asymptotic expression for ERF(v)

    INTEGER     , PARAMETER :: nmax = 10
    REAL(KIND=r8),PARAMETER :: epsilon= 0.02_r8
    REAL(KIND=r8),PARAMETER :: Omega0= 2.0_r8 ! where omega is a specific cutoff value for gama/ksqrt(2)

    REAL(KIND=r8),PARAMETER :: min_mu  =  1.e-12_r8
    REAL(KIND=r8),PARAMETER :: min_Q = 1.e-12_r8 
    REAL(KIND=r8) :: Omega(nCols)
    REAL(KIND=r8) :: Coef_Miu(nCols)
    REAL(KIND=r8) :: qsat
    REAL(KIND=r8) :: delta(nCols)
    REAL(KIND=r8) :: beta(nCols) 
    REAL(KIND=r8) :: zu2
    REAL(KIND=r8) :: zv2
    REAL(KIND=r8) :: Coef_k(nCols)
    REAL(KIND=r8) :: aux(nCols)
    REAL(KIND=r8) :: coeff
    REAL(KIND=r8) :: BLOCK1
    REAL(KIND=r8) :: dist_gk
    REAL(KIND=r8) :: fprime
    REAL(KIND=r8) :: det
    REAL(KIND=r8) :: pi
    REAL(KIND=r8) :: u
    REAL(KIND=r8) :: v
    REAL(KIND=r8) :: erfcu
    REAL(KIND=r8) :: erfcv
    REAL(KIND=r8) :: Coef_k1
    REAL(KIND=r8) :: Coef_k2
    REAL(KIND=r8) :: erf
    REAL(KIND=r8) :: hsqrtlog_2
    REAL(KIND=r8) :: v2
    REAL(KIND=r8) :: sqrtpi
    REAL(KIND=r8) :: sqrt2
    REAL(KIND=r8) :: zx1
    REAL(KIND=r8) :: zx2
    REAL(KIND=r8) :: exdel
    ! lconv = true si le calcul a converge (entre autre si qsub < min_q)
    LOGICAL       :: lconv(nCols)
    LOGICAL       :: PTCONV(nCols,kMax)
    REAL(KIND=r8) :: RATQSC(nCols,kMax) !Normalized width of the distribution
    REAL(KIND=r8) :: R     (nCols,kMax)
    REAL(KIND=r8) :: RS    (nCols,kMax)
    REAL(KIND=r8) :: QSUB  (nCols,kMax)
    REAL(KIND=r8) :: qes(nCols,kMax) 
    REAL(KIND=r8) :: tsp(nCols,kMax) 
    INTEGER       :: i
    INTEGER       :: K
    INTEGER       :: n
    INTEGER       :: m

    CALL findsp (nCols,kMax, q3, t3, prsl, tsp, qes)

    DO  K = 1, kMax !500
        DO i=1,nCols ! vector
           R     (i,k)=  q3     (i,k)  !  R--------kMax-: Domain-averaged mixing ratio of total water 
           RS    (i,k)=  qes    (i,k)  !  RS-------kMax-: Mean saturation humidity mixing ratio within the gridbox
           QSUB  (i,k)=  gliqp  (i,k)+gicep(i,k)   !  QSUB-----kMax-: Mixing ratio of condensed water within clouds associated
                                       !  with SUBGRID-SCALE condensation processes (here, it is
                                       !  predicted by the convection scheme)
        END DO
    END DO
    !cdir arraycomb
    cldf  (1:nCols,1:kMax)=0.0_r8        ! cym
    ratqsc(1:nCols,1:kMax)=0.0_r8        ! Normalized width of the distribution
    ptconv(1:nCols,1:kMax)=.FALSE.       ! Point convect if = TRUE
    !cdir end arraycomb

    pi = ACOS(-1.0_r8)

    sqrtpi=SQRT(pi)

    sqrt2=SQRT(2.0_r8)
    !
    !                         _______
    !              1   /\    /       \
    !hsqrtlog_2 = ---    \  / log (2)
    !              2      \/
    !
    hsqrtlog_2=0.5_r8*SQRT(LOG(2.0_r8))

    DO  K = 1, kMax !500

       DO i=1,nCols ! vector
          Coef_Miu(i)    = R(i,K)           ! Domain-averaged mixing ratio of total water assumed to be a random variable characterized by a mean value
          Coef_Miu(i)    = MAX(Coef_Miu(i),min_mu)! Domain-averaged mixing ratio of total water assumed to be a random variable characterized by a mean value
          qsat     = RS(i,K)          ! Mean saturation humidity mixing ratio within the gridbox
          qsat     = MAX(qsat,min_mu) ! Mean saturation humidity mixing ratio within the gridbox
          !
          ! Log relative humidity
          !             _          _               _          _  
          !            |   X - miu  |             |      miu   | 
          ! gama = log | -----------| ~ delta =   | -----------| 
          !            |_  X - qs  _|             |_     qs   _| 
          !
          ! Hipothesis the the second and third statistical moments can be diagnosed 
          !  i) fronm the in-cloud water content predicted by the convection scheme as a resultof convective activity that place the domain
          ! ii) From the degree of saturation of the large scale  environmental, and 
          !iii) by insisting that the total water mixing ratio values ar posiive
          !     so X = 0
          !
          delta(i) = LOG(Coef_Miu(i)/qsat)  ! Napierian logarithm of the rate mixing ratio of total water  and  saturation mixing ratio within the gridbox (relative Humidty)
          !                                   enddo ! vector
          !
          ! ***          There is no subgrid-scale condensation;        ***
          ! ***   the scheme becomes equivalent to an "all-or-nothing"  *** 
          ! ***             large-scale condensation scheme.            ***
          !

          !
          ! ***     Some condensation is produced at the subgrid-scale       ***
          ! ***                                                              ***
          ! ***       PDF = generalized log-normal distribution (GNO)        ***
          ! ***   (k<0 because a lower bound is considered for the PDF)      ***
          ! ***                                                              ***
          ! ***  -> Determine x (the parameter k of the GNO PDF) such        ***
          ! ***  that the contribution of subgrid-scale processes to         ***
          ! ***  the in-cloud water content is equal to QSUB(K)              ***
          ! ***  (equations (13), (14), (15) + Appendix B of the paper)      ***
          ! ***  SANDRINE BONY  AND KERRYA. EMANUEL, November 2001,          ***
          ! ***  JOURNAL OF THE ATMOSPHERIC SCIENCES,                        ***
          ! ***  A Parameterization of the Cloudiness Associated with Cumulus***
          ! ***  Convection; EvaluationUsing TOGA COARE Data
          ! ***                                                              ***
          ! ***    Here, an iterative method is used for this purpose        ***
          ! ***    (other numerical methods might be more efficient)         ***
          ! ***                                                              ***
          ! ***          NB: the "error function" is called ERF              ***
          ! ***                 (ERF in double precision)                   ***
          !

          !  On commence par eliminer les cas pour lesquels on n'a pas
          !  suffisamment d'eau nuageuse.

          ! check the Mixing ratio of condensed water within clouds
          !  
          IF ( QSUB(i,K) .LT. min_Q ) THEN
             ! 
             !No Mixing ratio of condensed water within clouds
             !
             ptconv(i,k)=.FALSE. !Point convect if = TRUE
             ratqsc(i,k)=0.0_r8  !Normalized width of the distribution
             lconv(i)  = .TRUE.  ! calculation converges (among other things if qsub <min_q)
             !
             !   Rien on a deja initialise
             !
          ELSE 
             ! 
             !Yes have Mixing ratio of condensed water within clouds
             !
             lconv(i)  = .FALSE. !not  calculation converges (among other things if qsub <min_q)
             Omega (i)  =  Omega0  !   Omega0= 2.0_r8 -->v-value above which we use an asymptotic expression for ERF(v)
             !
             !
             ! delta= log(RH) === > Log relative humidity
             !
             !                     _          _  
             !        ql          |            | 
             !beta = ----  +  exp |  - log(RH) | 
             !        q           |_          _| 
             !
             !print*,delta(i),MIN(0.0_r8,delta(i)),Coef_Miu(i)/qsat, EXP( -MIN(0.0_r8,delta(i)) )
             !
             ! Eq. B1 (SANDRINE BONY  AND KERRYA. EMANUEL, November 2001)
             !
             beta(i) = QSUB(i,K)/Coef_Miu(i) + EXP( -MIN(0.0_r8,delta(i)) )
             !
             ! --  roots of equation v > Omega: v-value above which we use an asymptotic expression for ERF(v)
             ! delta = Log of relative humidity
             !
             det = delta(i) + Omega(i)*Omega(i)

             IF (det.LE.0.0_r8) Omega(i) = Omega0 + 1.0_r8

             det = delta(i) + Omega(i)*Omega(i)

             IF (det.LE.0.0_r8) THEN
                Coef_k(i) = -0.0001_r8
             ELSE 
                !
                !sqrt2=SQRT(2.0_r8)
                !
                !                          _______
                !                   /\    /       \
                !zx1=      -Omega *   \  /     2     
                !                      \/
                !
                zx1   = -sqrt2*Omega(i)
                !
                ! log(1) = 0
                !
                ! log(a + b) = log (a) + log (1 + b/a)
                !
                ! log(a - b) = log (a) + log (1 - b/a)
                !
                !                           ________________________
                !                  /\      /          log (RH)      \
                !zx2        =        \    / 1.0   + ----------------
                !                     \  /           Omega(i)*Omega(i)
                !                      \/
                !
                zx2   = SQRT(1.0_r8 + delta(i)/(Omega(i)*Omega(i)))
                !                          _______
                !                   /\    /       \
                !zx1=      -Omega *   \  /     2     
                !                      \/
                !
                !
                !                           ________________________
                !                  /\      /          log (RH)      \
                !zx2        =        \    / 1.0   + ----------------
                !                     \  /           Omega(i)*Omega(i)
                !                      \/
                !
                ! Coef_k1   = zx1*(1.0_r8 - zx2)
                !
                ! Coef_k1   = zx1 - zx1*zx2)
                !                          _______                      _______                  ________________________
                !                   /\    /       \              /\    /       \       /\      /             log (RH)    \
                !Coef_k1=  -Omega *   \  /     2        +Omega *   \  /     2     *      \    / 1.0   + ----------------
                !                      \/                           \/                    \  /          Omega(i)*Omega(i)
                !                                                                          \/
                !
                !                          _______                   ____________________________
                !                   /\    /       \         /\      /                            \
                !Coef_k1=  -Omega *   \  /     2        +     \    / 2.0 Omega(i)^2  + 2* log (RH)
                !                      \/                      \  /                      
                !                                               \/
                !

                Coef_k1   = zx1*(1.0_r8 - zx2)

                Coef_k2   = zx1*(1.0_r8 + zx2)

                Coef_k(i) = 1.01_r8 * Coef_k1

                IF ( Coef_k1 .GE. 0.0_r8 ) Coef_k(i) = 0.5_r8*Coef_k2

             ENDIF
             !                         _______
             !              1   /\    /       \
             !hsqrtlog_2 = ---    \  / log (2)
             !              2      \/
             !
             IF (delta(i).LT.0.0_r8) Coef_k(i) = -hsqrtlog_2

          ENDIF

       ENDDO       ! vector

       !----------------------------------------------------------------------
       !   Debut des nmax iterations pour trouver la solution.
       !----------------------------------------------------------------------

       DO n = 1, nmax 

          DO i=1,nCols ! vector
             IF (.NOT.lconv(i)) THEN

                ! if .lconv(i) ==false the calculation converges (among other things if qsub <min_q)

                u = delta(i)/(Coef_k(i)*sqrt2) + Coef_k(i)/(2.0_r8*sqrt2)

                v = delta(i)/(Coef_k(i)*sqrt2) - Coef_k(i)/(2.0_r8*sqrt2)

                v2 = v*v

                IF ( v .GT. Omega(i) ) THEN 

                   IF (     ABS(u)  .GT. Omega(i) &
                        .AND.  delta(i) .LT. 0.0_r8 ) THEN
                      !
                      ! -- use asymptotic expression of erf for u and v large:
                      ! ( -> analytic solution for Coef_k )
                      !
                      exdel=beta(i)*EXP(delta(i))
                      !
                      aux(i) = 2.0_r8*delta(i)*(1.0_r8-exdel)/(1.0_r8+exdel)
                      !
                      IF (aux(i).LT.0.0_r8) THEN
                         !                print*,'AUX(',i,',',k,')<0',aux(i),delta(i),beta(i)
                         aux(i)=0.0_r8
                      ENDIF
                      Coef_k(i) = -SQRT(aux(i))
                      !
                      !Eq. 13 of the SANDRINE BONY  AND KERRYA. EMANUEL, November 2001, 
                      !
                      BLOCK1 = EXP(-v*v) / v / sqrtpi
                      !
                      dist_gk   = 0.0_r8
                      fprime    = 1.0_r8
                      !
                   ELSE
                      !
                      ! -- erfv -> 1.0_r8, use an asymptotic expression of erfv for v large:
                      !
                      erfcu = 1.0_r8 - erf_cldf(u)
                      !
                      !  !!! ATTENTION : rajout d'un seuil pour l'exponentiel
                      aux(i) = sqrtpi*erfcu*EXP(MIN(v2,100.0_r8))
                      coeff  = 1.0_r8 - 0.5_r8/(v2) + 0.75_r8/(v2*v2)
                      !
                      !Eq. 12 of the SANDRINE BONY  AND KERRYA. EMANUEL, November 2001, 
                      !
                      BLOCK1 = coeff * EXP(-v2) / v / sqrtpi
                      !
                      dist_gk = v * aux(i) / coeff - beta(i)
                      !
                      fprime = 2.0_r8 / Coef_k(i) * (v2)   &
                           * ( EXP(-delta(i)) - u * aux(i) / coeff ) &
                           / coeff
                      !
                   ENDIF ! ABS(u)

                ELSE
                   !
                   ! -- general case:
                   !
                   erfcu = 1.0_r8-erf_cldf(u)
                   erfcv = 1.0_r8-erf_cldf(v)
                   !
                   !Eq. 13 of the SANDRINE BONY  AND KERRYA. EMANUEL, November 2001, 
                   !
                   BLOCK1 = erfcv

                   dist_gk = erfcu / erfcv - beta(i)
                   !
                   zu2=u*u
                   zv2=v2
                   IF(zu2.GT.20.0_r8.OR. zv2.GT.20.0_r8) THEN
                      !              print*,'ATTENTION !!! Coef_k(',i,') =', Coef_k(i)
                      !           print*,'ATTENTION !!! nCols,ND,R,RS,QSUB,PTCONV,RATQSC,CLDF',
                      !     .nCols,ND,R(i,k),RS(i,k),QSUB(i,k),PTCONV(i,k),RATQSC(i,k),
                      !     .CLDF(i,k)
                      !              print*,'ATTENTION !!! zu2 zv2 =',zu2(i),zv2(i)
                      zu2=20.0_r8
                      zv2=20.0_r8
                      fprime = 0.0_r8
                   ELSE
                      fprime = 2.0_r8 /sqrtpi/Coef_k(i) /(erfcv*erfcv) &
                             * (   erfcv*v*EXP(-zu2) - erfcu*u*EXP(-zv2) )
                   ENDIF
                ENDIF ! x

                ! -- test numerical convergence:

                !          if (beta(i).lt.1.e-10) then
                !              print*,'avant test ',i,k,lconv(i),u(i),v(i),beta(i)
                !              stop
                !          endif
                IF (ABS(fprime).LT.1.e-11_r8) THEN
                   !              print*,'avant test fprime<.e-11 '
                   !     s        ,i,k,lconv(i),u(i),v(i),beta(i),fprime(i)
                   !              print*,'nCols,ND,R,RS,QSUB',
                   !     s        nCols,ND,R(i,k),rs(i,k),qsub(i,k)
                   fprime=SIGN(1.e-11_r8,fprime)
                ENDIF


                IF ( ABS(dist_gk/beta(i)) .LT. epsilon ) THEN 
                   !           print*,'v-u **2',(v(i)-u(i))**2
                   !           print*,'exp v-u **2',exp((v(i)-u(i))**2)
                   ptconv(i,K) = .TRUE.  ! Point convect if = TRUE
                   lconv(i)=.TRUE.
                   !  borne pour l'exponentielle
                   !
                   ! ratqsc, Normalized width of the distribution
                   !
                   ratqsc(i,k)=MIN(2.0_r8*(v-u)*(v-u),20.0_r8)  
                   ratqsc(i,k)=SQRT(EXP(ratqsc(i,k))-1.0_r8)
                   CLDF(i,K) = MIN(MAX(0.5_r8 * BLOCK1,0.01_r8),1.0_r8)
                   IF( CLDF(i,K) == 0.01_r8)CLDF(i,K) =0.0_r8
                ELSE
                   Coef_k(i) = Coef_k(i) - dist_gk/fprime
                ENDIF
                !         print*,'apres test ',i,k,lconv(i)

             ENDIF ! lconv
          ENDDO       ! vector

          !----------------------------------------------------------------------
          !   Fin des nmax iterations pour trouver la solution.
       ENDDO ! n
       !----------------------------------------------------------------------

    END DO !500    CONTINUE  ! K

    RETURN
  END SUBROUTINE CLOUDS_GNO


SUBROUTINE CloudFractionLayrson(grh,gliq ,gice,frcliq,frcice)
 IMPLICIT  NONE
 !
 ! LOCAL VARIABEL
 !
 REAL(KIND=r8), INTENT(IN   ) :: grh
 REAL(KIND=r8), INTENT(IN   ) :: gliq 
 REAL(KIND=r8), INTENT(IN   ) :: gice
 REAL(KIND=r8), INTENT(OUT  ) :: frcliq
 REAL(KIND=r8), INTENT(OUT  ) :: frcice
 INTEGER, PARAMETER :: nVegClass=15
 INTEGER, PARAMETER :: nsoilay=1000
 INTEGER, PARAMETER :: ibMax=1
 INTEGER, PARAMETER :: jbMax=1

 REAL(KIND=r8), PARAMETER    :: beta_lw_canopy(nVegClass)     = RESHAPE ( (/ &
 !  beta_lw_canopy        !                                                  beta_lw_canopy
   1.300_r8, &    !  1: tropical evergreen forest / woodland                 0.962
   1.800_r8, &    !  2: tropical deciduous forest / woodland                 0.961
   2.000_r8, &    !  3: temperate evergreen broadleaf forest / woodland      0.966
   2.700_r8, &    !  4: temperate evergreen conifer forest / woodland        0.966
   2.100_r8, &    !  5: temperate deciduous forest / woodland                0.965
   1.500_r8, &    !  6: boreal evergreen forest / woodland                   0.960
   1.400_r8, &    !  7: boreal deciduous forest / woodland                   0.950
   1.800_r8, &    !  8: mixed forest / woodland                              0.960
   1.300_r8, &    !  9: savanna                                              0.962
   1.000_r8, &    ! 10: grassland / steppe                                   0.952
   1.200_r8, &    ! 11: dense shrubland                                      0.970
   1.100_r8, &    ! 12: open shrubland                                       0.950
   1.000_r8, &    ! 13: tundra                                               0.914
   2.000_r8, &    ! 14: desert                                               0.970
   1.900_r8  &    ! 15: polar desert / rock / ice                            0.970
   /), (/15/) )!---->  grassland / shrub systems  

 REAL(KIND=r8), PARAMETER    :: beta_up_canopy(nVegClass)     = RESHAPE ( (/ &
  !  beta_up_canopy        !                                                 beta_up_canopy
   1.300_r8, &    !  1: tropical evergreen forest / woodland                 0.962
   2.300_r8, &    !  2: tropical deciduous forest / woodland                 0.961
   2.300_r8, &    !  3: temperate evergreen broadleaf forest / woodland      0.966
   2.400_r8, &    !  4: temperate evergreen conifer forest / woodland        0.966
   2.200_r8, &    !  5: temperate deciduous forest / woodland                0.965
   1.300_r8, &    !  6: boreal evergreen forest / woodland                   0.960
   1.300_r8, &    !  7: boreal deciduous forest / woodland                   0.950
   2.500_r8, &    !  8: mixed forest / woodland                              0.960
   1.500_r8, &    !  9: savanna                                              0.962
   1.200_r8, &    ! 10: grassland / steppe                                   0.952
   1.200_r8, &    ! 11: dense shrubland                                      0.970
   1.100_r8, &    ! 12: open shrubland                                       0.950
   1.000_r8, &    ! 13: tundra                                               0.914
   2.100_r8, &    ! 14: desert                                               0.970
   2.000_r8  &    ! 15: polar desert / rock / ice                            0.970
   /), (/15/) )!---->  grassland / shrub systems  

 REAL(KIND=r8), PARAMETER    :: theta_lw_liquid_RH(nVegClass)     = RESHAPE ( (/ &
  !  theta_lw_canopy        !                                                theta_lw_canopy
    0.80_r8, &    !  1: tropical evergreen forest / woodland                 0.962
    40.0_r8, &    !  2: tropical deciduous forest / woodland                 0.961
    45.0_r8, &    !  3: temperate evergreen broadleaf forest / woodland      0.966
    33.0_r8, &    !  4: temperate evergreen conifer forest / woodland        0.966
    28.0_r8, &    !  5: temperate deciduous forest / woodland                0.965
    29.0_r8, &    !  6: boreal evergreen forest / woodland                   0.960
    27.0_r8, &    !  7: boreal deciduous forest / woodland                   0.950
    28.0_r8, &    !  8: mixed forest / woodland                              0.960
    50.0_r8, &    !  9: savanna                                              0.962
    18.0_r8, &    ! 10: grassland / steppe                                   0.952
    40.0_r8, &    ! 11: dense shrubland                                      0.970
    40.0_r8, &    ! 12: open shrubland                                       0.950
    15.0_r8, &    ! 13: tundra                                               0.914
    13.0_r8, &    ! 14: desert                                               0.970
    13.0_r8  &    ! 15: polar desert / rock / ice                            0.970
   /), (/15/) )!---->  grassland / shrub systems  

 REAL(KIND=r8), PARAMETER    :: theta_up_liquid_RH(nVegClass)     = RESHAPE ( (/ &
  !  theta_up_canopy        !                                                theta_up_canopy
   0.80_r8, &    !  1: tropical evergreen forest / woodland                 0.962
   150.0_r8, &    !  2: tropical deciduous forest / woodland                 0.961
   150.0_r8, &    !  3: temperate evergreen broadleaf forest / woodland      0.966
   120.0_r8, &    !  4: temperate evergreen conifer forest / woodland        0.966
    60.0_r8, &    !  5: temperate deciduous forest / woodland                0.965
    35.0_r8, &    !  6: boreal evergreen forest / woodland                   0.960
    40.0_r8, &    !  7: boreal deciduous forest / woodland                   0.950
    50.0_r8, &    !  8: mixed forest / woodland                              0.960
    70.0_r8, &    !  9: savanna                                              0.962
    23.0_r8, &    ! 10: grassland / steppe                                   0.952
    60.0_r8, &    ! 11: dense shrubland                                      0.970
    50.0_r8, &    ! 12: open shrubland                                       0.950
    16.0_r8, &    ! 13: tundra                                               0.914
    13.0_r8, &    ! 14: desert                                               0.970
    13.0_r8  &    ! 15: polar desert / rock / ice                            0.970
   /), (/15/) )!---->  grassland / shrub systems  

 REAL(KIND=r8), PARAMETER    :: delta_lw_canopy(nVegClass)     = RESHAPE ( (/ &
  !  delta_lw_canopy        !                                                delta_lw_canopy
   0.0e-0_r8, &    !  1: tropical evergreen forest / woodland                 0.962
   1.000_r8, &    !  2: tropical deciduous forest / woodland                 0.961
   1.000_r8, &    !  3: temperate evergreen broadleaf forest / woodland      0.966
   1.000_r8, &    !  4: temperate evergreen conifer forest / woodland        0.966
   1.000_r8, &    !  5: temperate deciduous forest / woodland                0.965
   1.000_r8, &    !  6: boreal evergreen forest / woodland                   0.960
   1.000_r8, &    !  7: boreal deciduous forest / woodland                   0.950
   1.000_r8, &    !  8: mixed forest / woodland                              0.960
   1.000_r8, &    !  9: savanna                                              0.962
   1.000_r8, &    ! 10: grassland / steppe                                   0.952
   1.000_r8, &    ! 11: dense shrubland                                      0.970
   1.000_r8, &    ! 12: open shrubland                                       0.950
   1.000_r8, &    ! 13: tundra                                               0.914
   1.000_r8, &    ! 14: desert                                               0.970
   1.000_r8  &    ! 15: polar desert / rock / ice                            0.970
   /), (/15/) )!---->  grassland / shrub systems  

 REAL(KIND=r8), PARAMETER    :: delta_up_canopy(nVegClass)     = RESHAPE ( (/ &
  !  delta_up_canopy        !                                                delta_up_canopy
   0.000_r8, &    !  1: tropical evergreen forest / woodland                 0.962
   1.000_r8, &    !  2: tropical deciduous forest / woodland                 0.961
   1.000_r8, &    !  3: temperate evergreen broadleaf forest / woodland      0.966
   1.000_r8, &    !  4: temperate evergreen conifer forest / woodland        0.966
   1.000_r8, &    !  5: temperate deciduous forest / woodland                0.965
   1.000_r8, &    !  6: boreal evergreen forest / woodland                   0.960
   1.000_r8, &    !  7: boreal deciduous forest / woodland                   0.950
   1.000_r8, &    !  8: mixed forest / woodland                              0.960
   1.000_r8, &    !  9: savanna                                              0.962
   1.000_r8, &    ! 10: grassland / steppe                                   0.952
   1.000_r8, &    ! 11: dense shrubland                                      0.970
   1.000_r8, &    ! 12: open shrubland                                       0.950
   1.000_r8, &    ! 13: tundra                                               0.914
   1.000_r8, &    ! 14: desert                                               0.970
   1.000_r8  &    ! 15: polar desert / rock / ice                            0.970
   /), (/15/) )!---->  grassland / shrub systems  
 REAL(KIND=r8)    :: x,maxdepth
 INTEGER :: i,j,k,inveg,lrec
 REAL(KIND=r8)    :: ice_ref(1:nsoilay)
 REAL(KIND=r8)    :: liq_ref(1:nsoilay)
 REAL(KIND=r8)    :: totdepth     ! total soil depth
 REAL(KIND=r8)    :: frootnorm1   ! normalization factor for Jackson rooting profile,low
 REAL(KIND=r8)    :: frootnorm2   ! normalization factor for Jackson rooting profile, up 
 REAL(KIND=r8)    :: depth(1:nsoilay)
 REAL(KIND=r8)    :: frootm(nsoilay,2)
 REAL(KIND=r8)    :: theta_lw_canopy,theta_up_canopy
! ************************************************************************
! define rooting profiles
! ************************************************************************
!
! define rooting profiles based upon data published in:
!
! Jackson et al., 1996:  A global analysis of root distributions
! for terrestrial biomes, Oecologia, 108, 389-411.
!
! and
!
! Jackson et al., 1997:  A global budget for fine root biomass, 
! surface area, and nutrient contents, Proceedings of the National
! Academy of Sciences, 94, 7362-7366.
!
   inveg=1
   liq_ref(1)=1.0e-8_r8
   DO k = 2, nsoilay
      liq_ref(k) = liq_ref(k-1)  +  ((1e-05_r8 - 1.0e-10_r8)/nsoilay)
   END DO
   ice_ref(1)=1.0e-10_r8
   DO k = 2, nsoilay
      ice_ref(k) = ice_ref(k-1)  +  ((1e-04_r8 - 1.0e-10_r8)/nsoilay)
   END DO

   DO k = 1, nsoilay
      !
      IF (k.eq.1) THEN
          depth(k) = liq_ref(k)
      ELSE
          depth(k) = depth(k-1) + liq_ref(k)
      END IF
      ! beta_lw_canopy   o parametro de forma
      ! theta_lw_canopy  o parametro de escala 
      ! delta_lw_canopy  o parametro de localizao
      maxdepth= depth(k)
   END DO

   frootnorm1=0.0
   DO k = 1, nsoilay
      !
      IF (k.eq.1) THEN
          depth(k) = liq_ref(k)
      ELSE
          depth(k) = depth(k-1) + liq_ref(k)
      END IF

      ! beta_lw_canopy   o parametro de forma
      ! theta_lw_canopy  o parametro de escala 
      ! delta_lw_canopy  o parametro de localizao
      theta_lw_canopy = (1.0-theta_lw_liquid_RH(inveg))
      x=depth(k)
      IF( x < delta_lw_canopy (inveg))STOP 'ERROR depth < delta_lw_canopy (inveg)'
      frootnorm1 =frootnorm1 +  (beta_lw_canopy(inveg)/theta_lw_canopy)*(((x-delta_lw_canopy(inveg))/theta_lw_canopy)**(beta_lw_canopy(inveg)-1.0))*(exp(-(((x-delta_lw_canopy(inveg))/theta_lw_canopy)**beta_lw_canopy(inveg))))
   END DO

   
   frcliq=0.0
   DO k = 1, nsoilay
      !
         IF (k.eq.1) THEN
             depth(k) = liq_ref(k)
         ELSE
             depth(k) = depth(k-1) +liq_ref(k)
         END IF
      IF( depth(k) <= gliq)THEN

         ! beta_lw_canopy   o parametro de forma
         ! theta_lw_canopy  o parametro de escala 
         ! delta_lw_canopy  o parametro de localizao
         theta_lw_canopy = (1.0-(grh))
         x=depth(k)
         IF( x < delta_lw_canopy (inveg))STOP 'ERROR depth < delta_lw_canopy (inveg)'
         frootm(k,1) = (beta_lw_canopy(inveg)/theta_lw_canopy)*(((x-delta_lw_canopy(inveg))/theta_lw_canopy)**(beta_lw_canopy(inveg)-1.0))*(exp(-(((x-delta_lw_canopy(inveg))/theta_lw_canopy)**beta_lw_canopy(inveg))))

         frootm(k,1) = frootm(k,1) / frootnorm1
         frcliq=frootm(k,1)*100
      END IF
!
    END DO
    
   frootnorm2=0.0
   DO k = 1, nsoilay
      !
      IF (k.eq.1) THEN
          depth(k) = ice_ref(k)
      ELSE
          depth(k) = depth(k-1) + ice_ref(k)
      END IF

      ! beta_lw_canopy   o parametro de forma
      ! theta_lw_canopy  o parametro de escala 
      ! delta_lw_canopy  o parametro de localizao
      theta_up_canopy = (1.0-(grh ))
      x=depth(k)
      IF( x < delta_lw_canopy (inveg))STOP 'ERROR depth < delta_lw_canopy (inveg)'
      frootnorm2 =frootnorm2 +  (beta_up_canopy(inveg)/theta_up_canopy)*(((x-delta_up_canopy(inveg))/theta_up_canopy)**(beta_up_canopy(inveg)-1.0))*(exp(-(((x-delta_up_canopy(inveg))/theta_up_canopy)**beta_up_canopy(inveg))))
   END DO


   frcice=0.0
   DO k = 1, nsoilay
      !
         IF (k.eq.1) THEN
             depth(k) = ice_ref(k)
         ELSE
             depth(k) = depth(k-1) +ice_ref(k)
         END IF
      IF( depth(k) <= gice)THEN
         ! beta_lw_canopy   o parametro de forma
         ! theta_lw_canopy  o parametro de escala 
         ! delta_lw_canopy  o parametro de localizao
         theta_up_canopy = (1.0-(grh))
         x=depth(k)
         IF( x < delta_lw_canopy (inveg))STOP 'ERROR depth < delta_lw_canopy (inveg)'
         frootm(k,2) = (beta_up_canopy(inveg)/theta_up_canopy)*(((x-delta_up_canopy(inveg))/theta_up_canopy)**(beta_up_canopy(inveg)-1.0))*(exp(-(((x-delta_up_canopy(inveg))/theta_up_canopy)**beta_up_canopy(inveg))))

         frootm(k,2) = frootm(k,2) / frootnorm2
         frcice=frootm(k,2)*1000
      END IF
!
    END DO

!      IF( x < 0) THEN
!          Fx(i,j) = 0.0_r8
!      ELSE
!          Fx(i,j) = 1 - exp(-(((x-delta)/theta)**(beta_lw_canopy)))
!      END IF
 
!   DO k = 1, nsoilay
      !

!             PRINT*,'depth(k)=',depth(k),'frootm(i,k,1,j)=',frootm(k,1)


!   END DO
!
! return to main program
!
END SUBROUTINE CloudFractionLayrson

  REAL(KIND=r8) FUNCTION erf_cldf(x)
        ! # MS Fortran
        ! Error function from Numerical Recipes.
        ! erf(x) = 1 - erfc(x)

   IMPLICIT NONE

   REAL(KIND=r8) :: dumerfc, x
   REAL(KIND=r8) :: t, z


   z = abs(x)
   t = 1.0_r8 / ( 1.0_r8 + 0.5_r8 * z )

   dumerfc =       t * exp(-z * z - 1.26551223_r8 + t *          &
               ( 1.00002368_r8 + t * ( 0.37409196_r8 + t *            &
             ( 0.09678418_r8 + t * (-0.18628806_r8 + t *            &
                   ( 0.27886807_r8 + t * (-1.13520398_r8 + t *            &
             ( 1.48851587_r8 + t * (-0.82215223_r8 + t * 0.17087277_r8 )))))))))

   IF ( x.lt.0.0_r8 ) dumerfc = 2.0_r8 - dumerfc
     
   erf_cldf = 1.0_r8 - dumerfc

  END FUNCTION erf_cldf


END MODULE CloudFractionModel
!
!==========================================================================================================================
!
PROGRAM MAIN
 USE CloudFractionModel,Only: cldgn3,CloudFractionLayrson
 USE wv_saturation,Only: gestbl
  IMPLICIT NONE
  INTEGER, PARAMETER       :: r4 = SELECTED_REAL_KIND(6)  ! Kind for 32-bits Real Numbers
  INTEGER, PARAMETER       :: r8 = SELECTED_REAL_KIND(15) ! Kind for 64-bits Real Numbers
  INTEGER, PARAMETER       :: ntime=228
  INTEGER, PARAMETER       :: ncols =1               !==>  Number of colum
  INTEGER, PARAMETER       :: kmax  =27              !==>  Number of layer
  INTEGER, PARAMETER       :: nls   =5               !==>  Number of layers in the stratosphere.
  INTEGER, PARAMETER       :: lcnvl =2               !==>  the lowest layer index where non-convective clouds can occur
  INTEGER, PARAMETER       :: lthncl=80              !==>  Minimum depth in mb of non-zero low level cloud  constant
  REAL(KIND=r4) :: aux   (ncols,kMax)  
  REAL(KIND=r4) :: aux2  (ncols) 

  REAL(KIND=r8) :: prsl  (ncols,kMax)   !==> pressure   (Pa)
  REAL(KIND=r8) :: prsi  (ncols,kMax+1) !==> interface layer pressure   (Pa)
  REAL(KIND=r8) :: grh   (ncols,kmax)   !==> relative humidity  (0-1)
  REAL(KIND=r8) :: shmt  (ncols,kmax)   !==> specific humidity  (kg/kg)
  REAL(KIND=r8) :: omg   (ncols,kmax)   !==> vertical velocity  (cb/sec)
  REAL(KIND=r8) :: gtmp  (ncols,kmax)   !==> layer temperature (k)
  REAL(KIND=r8) :: gice  (nCols,kMax)   !==> ice concentration  (kg/kg)
  REAL(KIND=r8) :: gliq  (nCols,kMax)   !==> liquid concentration  (kg/kg)
  REAL(KIND=r8) :: cldfrac  (nCols,kMax)   !==> Cloud Fraction  (%)
  REAL(KIND=r8) :: frcliq(nCols,kMax)
  REAL(KIND=r8) :: frcice(nCols,kMax)
  REAL(KIND=r8) :: avetemp(nCols,kMax,ntime)
  REAL(KIND=r8) :: sumtemp(nCols,kMax)
  REAL(KIND=r8) :: es    (nCols,kMax)   !==> vapor pressure 
  REAL(KIND=r8) :: rs    (nCols,kMax)   !==> saturation specific humidity  (kg/kg)

  REAL(KIND=r8) :: prcpt (ncols)        !==> precipitation at each time step.(m)
  REAL(KIND=r8) :: convc (ncols)        !==> convective cloud cover in 3 hr. avrage
  REAL(KIND=r8) :: convt (ncols)        !==> ncols convective cloud top  (sigma layer)
  REAL(KIND=r8) :: convb (ncols)        !==> ncols convective cloud base (sigma layer)
  REAL(KIND=r8) :: css   (ncols,kmax)   !==> supersatuation cloud cover fraction
  REAL(KIND=r8) :: ccu   (ncols,kmax)   !==> convective cloud cover fraction
  REAL(KIND=r8) :: cdin  (ncols,kmax)   !==> Inversion clouds
  REAL(KIND=r8) :: cstc  (ncols,kmax)   !==> Saturation clouds
  REAL(KIND=r8) :: ccon  (ncols,kmax)   !==> Convection clouds 
  REAL(KIND=r8) :: cson  (ncols,kmax)   !==> Shallow convective clouds
  REAL(KIND=r8) :: auxl (ncols,kmax)   !==> Shallow convective clouds
  INTEGER i,k

  real(kind=r4) :: var1(ncols,kmax)
  real(kind=r4) :: var2(ncols,kmax)
  real(kind=r4) :: var3(ncols,kmax)

 prsi = RESHAPE(SOURCE=(/ 1000.00_r8, 975.00_r8, 950.00_r8, 925.00_r8, &
                           900.00_r8, 875.00_r8, 850.00_r8, 825.00_r8, &
                           800.00_r8, 775.00_r8, 750.00_r8, 700.00_r8, &
                           650.00_r8, 600.00_r8, 550.00_r8, 500.00_r8, &
                           450.00_r8, 400.00_r8, 350.00_r8, 300.00_r8, &
                           250.00_r8, 225.00_r8, 200.00_r8, 175.00_r8, &
                           150.00_r8, 125.00_r8, 100.00_r8,  50.00_r8 /), SHAPE = (/ncols,kMax+1/))


 
 prsi=prsi*100_r8
 DO k=1,kMax 
    DO i=1,nCols
       prsl(i,k)=(prsi(i,k)+prsi(i,k+1))/2
       grh(i,k)=grh(i,k)
    END DO
 END DO

  CALL Init()
  CALL Run()
  CALL Finalize()

CONTAINS
 SUBROUTINE Init()
  IMPLICIT NONE
  INTEGER  :: lrec
  INQUIRE(IOLENGTH=lrec)aux
  OPEN(1,FILE='dados_entrada/dado_20140401.dat',ACCESS='DIRECT',FORM='UNFORMATTED',RECL=lrec,&
       ACTION='READ',STATUS='OLD')


  CALL gestbl()
 END SUBROUTINE 


 SUBROUTINE run()
  IMPLICIT NONE
  REAL(KIND=r8) :: TEMPC 
  REAL(KIND=r8) :: PRESS 
  INTEGER :: it,irec,irec2,irec3,lrec,lrec2,ncount
  REAL(KIND=r8) :: undef=-999.99999
  INTEGER       :: nlon=1
  INTEGER       :: nlat=1
  CHARACTER(LEN=12) :: time0='00z01apr2014'
  CHARACTER(LEN=3) :: deltat='3hr'
  REAL(KIND=r8) :: lon0=-60.5
  REAL(KIND=r8) :: lat0=-3.25
  REAL(KIND=r8) :: dlon=1.8
  REAL(KIND=r8) :: dlat=1.8
  INTEGER       :: nvar=3






  INQUIRE(IOLENGTH=lrec)aux
  OPEN(31,FILE='dados_entrada/perfil_cf_IOP1.bin',ACCESS='DIRECT',FORM='UNFORMATTED',RECL=lrec,&
       ACTION='READ',STATUS='OLD')

  OPEN(32,FILE='dados_entrada/perfil_icw_IOP1.bin',ACCESS='DIRECT',FORM='UNFORMATTED',RECL=lrec,&
       ACTION='READ',STATUS='OLD')

  OPEN(33,FILE='dados_entrada/perfil_lcw_IOP1.bin',ACCESS='DIRECT',FORM='UNFORMATTED',RECL=lrec,&
       ACTION='READ',STATUS='OLD')

  OPEN(34,FILE='dados_entrada/perfil_t_IOP1.bin',ACCESS='DIRECT',FORM='UNFORMATTED',RECL=lrec,&
       ACTION='READ',STATUS='OLD')

  OPEN(35,FILE='dados_entrada/perfil_q_IOP1.bin',ACCESS='DIRECT',FORM='UNFORMATTED',RECL=lrec,&
       ACTION='READ',STATUS='OLD')

  OPEN(36,FILE='dados_entrada/perfil_omega_IOP1.bin',ACCESS='DIRECT',FORM='UNFORMATTED',RECL=lrec,&
       ACTION='READ',STATUS='OLD')

  INQUIRE(IOLENGTH=lrec2)aux2
  OPEN(37,file='dados_entrada/prec_IOP1_pwd.dat',ACCESS='DIRECT',FORM='UNFORMATTED',RECL=lrec2,&
       ACTION='READ',STATUS='OLD')

  open(2,file='dados_saida/cf_20140401.dat',access='direct',form='unformatted', status='unknown',&
       recl=lrec2) !4bytes*k(leves)*ncols*3(variables) para 1 camada

      !**********Temperature "K" (Eq. 2.64 do Wallace e Hobbs)***********

  sumtemp=0
  avetemp=0
  irec=0
  ncount=0
  DO it=1,ntime
      irec=irec+1
      READ(34,rec=irec)aux
      DO k=1,kMax
         DO i=1,nCols
            gtmp (i,k)=aux(i,k)
         END DO
      END DO
      IF(ncount <8 )THEN
         ncount=ncount+1
         DO k=1,kMax
            DO i=1,nCols
               if(it>1) avetemp(i,k,it)=avetemp(i,k,it-1)
               sumtemp(i,k)=sumtemp(i,k)+gtmp (i,k)
            END DO
         END DO
      ELSE
         DO k=1,kMax
            DO i=1,nCols
               avetemp(i,k,it)=sumtemp(i,k)/ncount
            END DO
         END DO
         ncount=1.0
         sumtemp=0.0
         DO k=1,kMax
            DO i=1,nCols
               sumtemp(i,k)=sumtemp(i,k)+gtmp (i,k)
            END DO
         END DO

      END IF
  END DO

  irec=0
  irec2=0
  irec3=0
  DO it=1,ntime
      irec=irec+1
      irec2=irec2+1
      !********** Cloud Fraction "%" ()***********
      READ(31,rec=irec)aux
      DO k=1,kMax
         DO i=1,nCols
            cldfrac(i,k)=aux(i,k)
         END DO
      END DO
      !********** Mixed Ratio ice water "kg/Kg" (Eq. 2.64 do Wallace e Hobbs)***********
      READ(32,rec=irec)aux
      DO k=1,kMax
         DO i=1,nCols
            gice (i,k)=aux(i,k)/1000.0
         END DO
      END DO
      !********** Mixed Ratio  liquid water "kg/Kg" (Eq. 2.64 do Wallace e Hobbs)***********
      READ(33,rec=irec)aux
      DO k=1,kMax
         DO i=1,nCols
            gliq (i,k)=aux(i,k)/1000.0
         END DO
      END DO
      !**********Temperature "K" (Eq. 2.64 do Wallace e Hobbs)***********
      READ(34,rec=irec)aux
      DO k=1,kMax
         DO i=1,nCols
            gtmp (i,k)=aux(i,k)
         END DO
      END DO
      !********** Relative Hunidity "%" (Eq. 2.64 do Wallace e Hobbs)***********
      READ(35,rec=irec)aux
      DO k=1,kMax
         DO i=1,nCols
            shmt(i,k)=aux(i,k)/1000.0_r8            !g/kg ---> kg/kg
            TEMPC=gtmp(i,k)-273.16_r8
            !********Pressao de vapor de saturacao "hPa" (Eq. 10 do Bolton, 1980)*******
            es(i,k)=6.112_r8*EXP(17.67*TEMPC/(243.5_r8 + TEMPC))     
            PRESS=prsl(i,k)/100.0_r8
            !**** Razao de mistura de saturacao "kg/Kg" (Eq. 2.64 do Wallace e Hobbs)****
            rs(i,k)=(es(i,k)*622.0_r8/(PRESS-es(i,k)))/1000.0_r8
            !************************* Umidade Relativa "%"*****************************
            grh(i,k)=shmt(i,k)/rs(i,k)
         END DO
      END DO
      !********** Omega "Pa/s" (Eq. 2.64 do Wallace e Hobbs)***********
      READ(36,rec=irec)aux
      DO k=1,kMax
         DO i=1,nCols
            !omg=((aux)/10.0_r8)/3600.0_r8  !  mb/hr --- > cb/sec

            omg (i,k)=((aux(i,k)/10.0_r8)/3600.0_r8)
         END DO
      END DO

      !********** Precipitation mm/h" (Eq. 2.64 do Wallace e Hobbs)***********
      READ(37,rec=irec2)aux2
      DO i=1,nCols
         !    Comvert (mm/hr)  to (m) 
         prcpt(i)=(aux2(i)/1000.0)*3 ! convective precipitation [m]
      END DO

      PRINT*,'it=',it

      DO k=1,kMax
         DO i=1,nCols
             CALL CloudFractionLayrson(grh(i,k),gliq (i,k),gice(i,k),frcliq(i,k),frcice(i,k))
         END DO
      END DO
      DO i=1,nCols

      !READ(1,rec=irec)aux
       convt(i)=kMax-nls   ! index top cloud convective  
      !READ(1,rec=irec)aux
       convb(i)=2          ! index botton cloud convective 
      END DO

  
       CALL  cldgn3 (prsl (1:ncols,1:kMax)   , &!REAL(KIND=r8),INTENT(in   ) :: prsl  (ncols,kMax)   ==> pressure        (Pa)
                     prsi (1:ncols,1:kMax+1) , &!REAL(KIND=r8),INTENT(in   ) :: prsi  (ncols,kMax+1) ==> interface layer pressure        (Pa)
                     shmt (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(in   ) :: grh   (ncols,kmax)   ==> specific humidity  (kg/kg)
                     gice (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(in   ) :: grh   (ncols,kmax)   ==> ice concentration  (kg/kg)
                     gliq (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(in   ) :: grh   (ncols,kmax)   ==> liquid concentration  (kg/kg)
                     grh  (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(in   ) :: grh   (ncols,kmax)   ==> relative humidity  (0-1)
                     omg  (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(in   ) :: omg   (ncols,kmax)   ==> vertical velocity  (cb/sec)
                     gtmp (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(in   ) :: gtmp  (ncols,kmax)   ==> layer temperature (k)
                     prcpt(1:ncols)          , &!REAL(KIND=r8),INTENT(in   ) :: prcpt (ncols)        ==> precipitation at each time step.(m)
                     convc(1:ncols)          , &!REAL(KIND=r8),INTENT(in   ) :: convc (ncols)        ==> convective cloud cover in 3 hr. avrage
                     convt(1:ncols)          , &!REAL(KIND=r8),INTENT(in   ) :: convt (ncols)        ==> ncols convective cloud top  (sigma layer)
                     convb(1:ncols)          , &!REAL(KIND=r8),INTENT(in   ) :: convb (ncols)        ==> ncols convective cloud base (sigma layer)
                     css  (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(inout) :: css   (ncols,kmax)   ==> supersatuation cloud cover fraction
                     ccu  (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(inout) :: ccu   (ncols,kmax)   ==> convective cloud cover fraction
                     cdin (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(inout) :: cdin  (ncols,kmax)   ==> Inversion clouds
                     cstc (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(inout) :: cstc  (ncols,kmax)   ==> Saturation clouds
                     ccon (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(inout) :: ccon  (ncols,kmax)   ==> Convection clouds 
                     cson (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(inout) :: cson  (ncols,kmax)   ==> Shallow convective clouds
                     auxl (1:ncols,1:kmax)   , &!REAL(KIND=r8),INTENT(inout) :: auxl  (ncols,kmax)   ==>  clouds
                     lcnvl                   , &!INTEGER      ,INTENT(in   ) :: lcnvl       ==>  the lowest layer index where non-convective clouds can occur
                     lthncl                  , &!INTEGER      ,INTENT(in   ) :: lthncl      ==>  Minimum depth in mb of non-zero low level cloud constant
                     ncols                   , &!INTEGER      ,INTENT(IN   ) :: ncols       ==>  Number of colum
                     kmax                    , &!INTEGER      ,INTENT(in   ) :: kmax        ==>  Number of layer
                     nls                       )!INTEGER      ,INTENT(in   ) :: nls         ==>  Number of layers in the stratosphere.


      DO k=1,kMax
         DO i=1,nCols
            PRINT*,cldfrac(i,k),frcliq(i,k)+frcice(i,k),auxl(i,k),(gtmp (i,k)-avetemp(i,k,it))/abs(gtmp (i,k)+avetemp(i,k,it))
         END DO
      END DO


!  print*, prsl
  DO k=1,kMax
     DO i=1,nCols
         var1(i,k) = real(gtmp(i,k)-avetemp(i,k,it))
         irec3=irec3+1
         write(2,rec=irec3) real(var1(i,k))
      END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var1(i,k) = real(shmt(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(shmt(i,k))
      END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var1(i,k) = real(omg(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(omg(i,k))
      END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var1(i,k) = real(es(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(es(i,k))
      END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var1(i,k) = real(grh(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(grh(i,k))
      END DO
  END DO
  DO k=1,1
     DO i=1,nCols
         irec3=irec3+1
         write(2,rec=irec3) real(prcpt(i))
      END DO
  END DO
  
  DO k=1,kMax
     DO i=1,nCols
         var2(i,k) = real(css(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(css(i,k))
       END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var3(i,k) = real(ccu(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(ccu(i,k))
     END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var3(i,k) = real(cdin(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(cdin(i,k))
     END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var3(i,k) = real(cstc(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(cstc(i,k))
     END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var3(i,k) = real(ccon(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(ccon(i,k))
     END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var3(i,k) = real(cson(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(cson(i,k))
     END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var3(i,k) = real(frcliq(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(var3(i,k))
     END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var3(i,k) = real(frcice(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(var3(i,k))
     END DO
  END DO
  DO k=1,kMax
     DO i=1,nCols
         var3(i,k) = real(cldfrac(i,k))
         irec3=irec3+1
         write(2,rec=irec3) real(var3(i,k))
     END DO
  END DO

  END DO


  print*,'close file'
  close(2)

  OPEN(12,file='dados_saida/cf_20140401.ctl',access='sequential',form='formatted', status='unknown',&
       action='WRITE')

 nvar=15
WRITE(12,'(A6,A               )')'dset  ','^cf_20140401.dat'
WRITE(12,'(A6,F12.5           )')'undef ',undef
WRITE(12,'(A6,I6,A8,2F12.4    )')'xdef  ',nlon ,' linear ',lon0 ,dlon
WRITE(12,'(A6,I6,A8,2F12.4    )')'ydef  ',nlat ,' linear ',lat0 ,dlat
WRITE(12,'(A6,I6,A8,A12,A,A3  )')'tdef  ',ntime,' linear ',time0,'  ', deltat
WRITE(12,'(A6,I6,A8,27F10.2   )')'zdef  ',kMax ,' levels ',(prsl(1,k),k=1,kMax)
WRITE(12,'(A6,I6              )')'VARS  ',nvar
WRITE(12,'(A6,I6,A            )')'gtmp  ',kMax,' 99 absolute temperature '
WRITE(12,'(A6,I6,A            )')'shmt  ',kMax,' 99 specific humidity '
WRITE(12,'(A6,I6,A            )')'omg   ',kMax,' 99 vertical velocity '
WRITE(12,'(A6,I6,A            )')'es    ',kMax,' 99 vapor pressure'
WRITE(12,'(A6,I6,A            )')'grh   ',kMax,' 99 relativa humidity '
WRITE(12,'(A6,I6,A            )')'prec  ',kMax-kMax,' 99 precipitation  '
WRITE(12,'(A6,I6,A            )')'css   ',kMax,' 99 supersatuation cloud cover fraction      '
WRITE(12,'(A6,I6,A            )')'ccu   ',kMax,' 99 convective cloud cover fraction          '
WRITE(12,'(A6,I6,A            )')'cdin  ',kMax,' 99  Inversion clouds'
WRITE(12,'(A6,I6,A            )')'cstc  ',kMax,' 99  Saturation clouds'
WRITE(12,'(A6,I6,A            )')'ccon  ',kMax,' 99  Convection clouds '
WRITE(12,'(A6,I6,A            )')'cson  ',kMax,' 99  Shallow convective clouds'
WRITE(12,'(A6,I6,A            )')'fliq  ',kMax,' 99  Shallow convective clouds'
WRITE(12,'(A6,I6,A            )')'fice  ',kMax,' 99  Shallow convective clouds'
WRITE(12,'(A6,I6,A            )')'ftot  ',kMax,' 99  Shallow convective clouds'
WRITE(12,'(A7                 )')'ENDVARS'

!---------------------------------------------------------------------------------------------------

 END SUBROUTINE 
 SUBROUTINE Finalize()
  IMPLICIT NONE

 END SUBROUTINE
END PROGRAM MAin
